/***************************************************************************
 * ISDCUST.H - Customer Header File
 *                                                                          
 * Copyright (C) 1998-2000 In-System Design, Inc.                                
 */                                                                         

/* define contants that can be customized */
#define DEFAULT_INTERFACE			0
#define DEFAULT_ALTINTERFACE		2

#define ATA_BYTES_PER_BLOCK 512

#define IRP_INFINITE 	MAXLONG
#define IRP_TIMEOUT 	30000
#define CDB_TIMEOUT 	5000
#define IOCTL_TIMEOUT 	3000
#define SEQ_TIMEOUT 	3000
#define DEFAULT_DELAY 	5

#define TAPE_CMD_TIMEOUT_MS 		360
#define CDROM_CMD_TIMEOUT_MS 		180
#define OPTICAL_CMD_TIMEOUT_MS 		40
#define STD_CMD_TIMEOUT_MS 			30

/* custom FDEVICE_EXTENSION->DriverFlags state flags */
/*     (use high order word only)                    */
#define DF_CUST_FALSTAFFLITE		0x00010000
#define DF_CUST_UDRIVE 				0x00020000
#define DF_CUST_APM_SUPPORT 		0x00040000

//
// ATAPI State Machine constants
//
#define ATAPI_STATE_COMMAND_START		 					0x00
#define ATAPI_STATE_DEVICE_BUSY			 					0x01
#define ATAPI_STATE_CHECK_CYLHILO 							0x02
#define ATAPI_STATE_DATA_OPTIMIZED							0x03
#define ATAPI_STATE_DATA_LOOP_ON_DRQ						0x04
#define ATAPI_STATE_DATA_NOT_FINISHED						0x05
#define ATAPI_STATE_DO_CYLHILO								0x06
#define ATAPI_STATE_COMPLETE_TIMEOUT 						0x07
#define ATAPI_STATE_CHECK_COMMAND_COMPLETE					0x08
#define ATAPI_STATE_DEVICE_ERROR		 					0xFD
#define ATAPI_STATE_RESET_ERROR			 					0xFE
#define ATAPI_STATE_COMMAND_COMPLETE	 					0xFF

//
// ATA return state constants
//
#define ATA_STATE_SUCCESS			 						0x00
#define ATA_STATE_RESUBMIT			 						0x01
#define ATA_STATE_TIMEOUT									0x02
#define ATA_STATE_INVALID_REQUEST							0xFD
#define ATA_STATE_DEVICE_ERROR		 						0xFE
#define ATA_STATE_RESET_ERROR			 					0xFF

//
// WAIT return state constants
//
#define WAIT_STATE_SUCCESS			 						0x00
#define WAIT_STATE_BUSY				 						0x01
#define WAIT_STATE_TIMEOUT_ERROR		 					0xFC
#define WAIT_STATE_DEVICE_DRQ 								0xFD
#define WAIT_STATE_DEVICE_ERROR		 						0xFE
#define WAIT_STATE_RESET_ERROR			 					0xFF

#define ATA_REGMASK_FEATURES 			0x01
#define ATA_REGMASK_SECTOR_COUNT 		0x02
#define ATA_REGMASK_SECTOR_NUM 			0x04
#define ATA_REGMASK_CYLINDER 			0x08
#define ATA_REGMASK_HEAD				0x10

#define ATA_CMDMASK_NO_CHECK_FOR_BSY_AND_ERR_AFTER_CMD 		0x01
#define ATA_CMDMASK_CHECK_FOR_DRQ_AFTER_CMD 				0x02
#define ATA_CMDMASK_WILL_XFER_DATA							0x04


// CDB operation codes not normally defined
#define SCSIOP_WRITE12             0xAA
#define SCSIOP_WRITECONTINUE       0xE1

//
// IDE constants
//

#define ATA_ADDRESS_ALT_STATUS  0x0e
#define ATA_ADDRESS_COMMAND     0x17
#define ATA_ADDRESS_CYL_HIGH    0x15
#define ATA_ADDRESS_CYL_LOW     0x14
#define ATA_ADDRESS_DATA        0x10
#define ATA_ADDRESS_DEV_HEAD    0x16
#define ATA_ADDRESS_DEV_CNTRL   0x0e
#define ATA_ADDRESS_ERROR       0x11
#define ATA_ADDRESS_FEATURES    0x11
#define ATA_ADDRESS_SECT_CNT    0x12
#define ATA_ADDRESS_SECT_NUM    0x13
#define ATA_ADDRESS_STATUS      0x17

//
// ATAPI address aliases
//
#define ATAPI_ADDRESS_DATA              (ATA_ADDRESS_DATA)
#define ATAPI_ADDRESS_INTR_REASON       (ATA_ADDRESS_SECT_CNT)
#define ATAPI_ADDRESS_BYTE_COUNT_LOW    (ATA_ADDRESS_CYL_LOW)
#define ATAPI_ADDRESS_BYTE_COUNT_HIGH   (ATA_ADDRESS_CYL_HIGH)
#define ATAPI_ADDRESS_DRIVE_SELECT      (ATA_ADDRESS_DEV_HEAD)
#define ATAPI_ADDRESS_COMMAND           (ATA_ADDRESS_COMMAND)

//
// ATA_ADDRESS_STATUS (and ATA_ADDRESS_ALT_STATUS) definition
//
#define ATA_STATUS_ERROR             0x01
#define ATA_STATUS_INDEX             0x02
#define ATA_STATUS_CORRECTED_ERROR   0x04
#define ATA_STATUS_DRQ               0x08
#define ATA_STATUS_DSC               0x10
#define ATA_STATUS_DEVICE_FAULT      0x20
#define ATA_STATUS_DRDY              0x40
#define ATA_STATUS_IDLE              0x50
#define ATA_STATUS_BUSY              0x80

//
// ATA_ADDRESS_DEV_HEAD definition
//
#define ATA_ADDRESS_DEVHEAD_STD         0xa0 // on-bits required for all head-reg writes
#define ATA_ADDRESS_DEVHEAD_LBA_MODE    0x40    
#define ATA_ADDRESS_DEVHEAD_SLAVE       0x10

//
// ATA commands
//
#define ATA_COMMAND_ATAPI_RESET      		0x08
#define ATA_COMMAND_RECALIBRATE      		0x10
#define ATA_COMMAND_READ             		0x20
#define ATA_COMMAND_WRITE            		0x30
#define ATA_COMMAND_VERIFY           		0x40
#define ATA_COMMAND_SEEK             		0x70
#define ATA_COMMAND_SET_DRIVE_PARAMETERS 	0x91
#define ATA_COMMAND_READ_MULTIPLE    		0xC4
#define ATA_COMMAND_WRITE_MULTIPLE   		0xC5
#define ATA_COMMAND_SET_MULTIPLE     		0xC6
#define ATA_COMMAND_READ_DMA         		0xC8
#define ATA_COMMAND_WRITE_DMA             	0xCA
#define ATA_COMMAND_GET_MEDIA_STATUS      	0xDA
#define ATA_COMMAND_STANDBY				   	0xE2
#define ATA_COMMAND_SET_FEATURES		   	0xEF
#define ATA_COMMAND_IDENTIFY              	0xEC
#define ATA_COMMAND_MEDIA_EJECT           	0xED
#define ATA_COMMAND_DOOR_LOCK               0xDE // prevent media removal
#define ATA_COMMAND_DOOR_UNLOCK             0xDF // allow media removal

//
// ATAPI commands
//
#define ATAPI_COMMAND_MODE_SENSE            0x5A
#define ATAPI_COMMAND_MODE_SELECT           0x55
#define ATAPI_COMMAND_FORMAT_UNIT           0x24
#define ATAPI_COMMAND_PACKET                0xA0
#define ATAPI_COMMAND_IDENTIFY              0xA1

//
// ATA features values (for ATA_COMMAND_SET_FEATURES)
//
#define ATA_FEATURE_ENABLE_WRITE_CACHE                      0x02
#define ATA_FEATURE_SET_TRANSFER_MODE                       0x03
#define ATA_FEATURE_ENABLE_ADV_POWER                        0x05
#define ATA_FEATURE_APPEND_DATA_TO_RW_LONG                  0x44
#define ATA_FEATURE_DISABLE_READ_AHEAD                      0x55
#define ATA_FEATURE_DISABLE_REVERTING_TO_POWER_ON_DEFAULTS  0x66
#define ATA_FEATURE_DISABLE_WRITE_CACHE                     0x82
#define ATA_FEATURE_ENABLE_REVERTING_TO_POWER_ON_DEFAULTS   0xcc
#define ATA_FEATURE_ENABLE_MEDIA_STATUS                     0x95


/* ATA drive control definitions */
#define ATA_DC_DISABLE_INTERRUPTS    0x02
#define ATA_DC_RESET_CONTROLLER      0x04
#define ATA_DC_REENABLE_CONTROLLER   0x00

/* ATA error definitions */
#define ATA_ERROR_BAD_BLOCK          0x80
#define ATA_ERROR_DATA_ERROR         0x40
#define ATA_ERROR_MEDIA_CHANGE       0x20
#define ATA_ERROR_ID_NOT_FOUND       0x10
#define ATA_ERROR_MEDIA_CHANGE_REQ   0x08
#define ATA_ERROR_COMMAND_ABORTED    0x04
#define ATA_ERROR_END_OF_MEDIA       0x02
#define ATA_ERROR_ILLEGAL_LENGTH     0x01

/* IDENTIFY capability bit definitions */
#define IDENTIFY_CAPABILITIES_DMA_SUPPORTED 0x0100
#define IDENTIFY_CAPABILITIES_LBA_SUPPORTED 0x0200

/* IDENTIFY DMA timing cycle mode */
#define IDENTIFY_DMA_CYCLES_MODE_0 0x00
#define IDENTIFY_DMA_CYCLES_MODE_1 0x01
#define IDENTIFY_DMA_CYCLES_MODE_2 0x02

/* ATAPI interrupt reasons */
#define ATAPI_IR_COD 0x01
#define ATAPI_IR_IO  0x02

#define IDENTIFY_GENERAL_CFGF_REMOVABLE_MEDIA   (0x0080)


/* define ATA Register Read buffer order */
#define REG_ALT_STATUS 				0x00
#define REG_ERROR					0x01
#define REG_CYLINDER_LOW			0x02
#define REG_CYLINDER_HIGH			0x03

/* define general register addresses */
#define REG_DATA 0x00
#define REG_STAT 0x01
#define REG_CTRL 0x02
#define REG_EPPA 0x03
#define REG_EPPD 0x04
#define REG_ECPA 0x05
#define REG_ECRR 0x06
#define REG_CCTR 0x07
#define REG_SETU 0x08
#define REG_TIMO 0x09
#define REG_ATAA 0x0A
#define NUMBER_OF_REGS 0x0B

/* define register addresses for read */
#define REG_STAT_R 0x00
#define REG_CTRL_R 0x01
#define REG_ECRR_R 0x02
#define REG_CCTR_R 0x03
#define REG_DATA_R 0x04
#define REG_EPPA_R 0x05
#define REG_EPPD_R 0x05
#define REG_SETU_R 0x06
#define REG_TIMO_R 0x07
#define REG_ATAA_R 0x08
#define NUMBER_OF_REGS_R 0x09

//
// REG_STAT register definition
//
#define REG_STAT_PPORTBUSY_OR_IOCS16    (0x80)
#define REG_STAT_PPORTACK_OR_IORDY      (0x40)
#define REG_STAT_PPORTERR_OR_PDIAG      (0x20)
#define REG_STAT_PPORTSEL_OR_INTRQ      (0x10)
#define REG_STAT_PPORTFAULT_OR_DASP     (0x08)
#define REG_STAT_1284_DISABLED          (0x04)
#define REG_STAT_PPORTPLH_OR_DMARQ      (0x02)
#define REG_STAT_EPP_TIMEOUT            (0x01)

//
// REG_SETU register definition
//
#define REG_SETU_HOLD_BULK              (0x80)
#define REG_SETU_TERMINATION_WAIT       (0x40)
#define REG_SETU_CLEAR_BULK_OUT         (0x20)
#define REG_SETU_FORCE_NEGOTIATION      (0x10)
#define REG_SETU_FORCE_COMPATIBILITY    (0x08)
#define REG_SETU_AUTO_HTR               (0x04)
#define REG_SETU_FILTER_SFT_OVERRIDE    (0x02)    
#define REG_SETU_FILTER_ENABLE          (0x01)

#define REG_SETU_STD                    (REG_SETU_HOLD_BULK|REG_SETU_FILTER_SFT_OVERRIDE)

//
// REG_CTRL register definition
//
#define REG_CTRL_PPORTHLH_RESET         (0x80)
#define REG_CTRL_EPP_MASK               (0x40)
#define REG_CTRL_DIRECTION_IN           (0x20) // else, is OUT
#define REG_CTRL_INT_ENABLE             (0x10)
#define REG_CTRL_PPORTSELECTIN_CSEL     (0x08)
#define REG_CTRL_PPORTNINIT_DMACK       (0x04)
#define REG_CTRL_PPORTAUTOFD_DIOR       (0x02)
#define REG_CTRL_PPORTSTROBE_DIOW       (0x01)

//
// REG_ECCR register definition
//
#define REG_ECCR_MODE_STANDARD          (0x00)
#define REG_ECCR_MODE_BIDIRECTIONAL     (0x20)
#define REG_ECCR_MODE_COMPATIBILITY     (0x40)
#define REG_ECCR_MODE_ECP               (0x60)
#define REG_ECCR_MODE_EPP               (0x80)
#define REG_ECCR_MODE_ATA               (0xE0)
#define REG_ECCR_NACK_INTR              (0x10)
#define REG_ECCR_NFAULT_INTR            (0x08)
#define REG_ECCR_BULKIN_INTR            (0x04)
#define REG_ECCR_BULKIN_EMPTY           (0x02)
#define REG_ECCR_BULKOUT_EMPTY          (0x01)

//
// REG_CCTR register definition
//
#define REG_CCTR_MANUAL_REVERSE         (0x80)
#define REG_CCTR_SLOW_RCCM              (0x40)
#define REG_CCTR_SLOW_INSTRUCTION       (0x20)
#define REG_CCTR_RETURN_ON_NO_DATA      (0x10)
#define REG_CCTR_NFAULT_INT_MASK        (0x08)
#define REG_CCTR_COMPRESS_ENABLE        (0x02)
#define REG_CCTR_AUTO_MODE              (0x01)

#define SEQ_RRMW 0x10
#define SEQ_RRCM 0x20
#define SEQ_RWRI 0xa0
#define SEQ_RREN 0x30
#define SEQ_WBIB 0x40
#define SEQ_WBOB 0x50
#define SEQ_DATI 0x60
#define SEQ_DATO 0x70
#define SEQ_EPPI 0x90
#define SEQ_NOOP 0x80
#define SEQ_SUBR 0xB0
#define SEQ_RETN 0xB0
#define SEQ_BLOCKING_BIT 0x80

#define GET_DEVICE_ID		0 
#define GET_PORT_STATUS		1 
#define SOFT_RESET			2 
#define GET_1284_REGISTER	3 
#define SET_1284_REGISTER	4
#define GET_SEQ_STATUS	 	5
#define SEQ_LOAD		 	6
#define SEQ_ABORT		 	7
#define LOAD_EEPROM		 	8
#define CLEAR_BUFFER	 	9

// SEQUENCER_STATUS.statusBits
#define SEQSTATUSF_ACTIVE                   0x80
#define SEQSTATUSF_STOPPED                  0x40
#define SEQSTATUSF_EXECUTING_SUBROUTINE     0x20

typedef struct _SEQUENCER_STATUS {
    UCHAR   seqPos;
    UCHAR   globalLoopCountHigh;
    UCHAR   globalLoopCountLow;
    UCHAR   statusFlags;
    UCHAR   instructLoopCountHigh;
    UCHAR   instructLoopCountLow;
} SEQUENCER_STATUS, *PSEQUENCER_STATUS;

typedef struct _COMPLETION_CONTEXT {
    PDEVICE_OBJECT DeviceObject;
    PKEVENT Event;
	PURB Urb;
	PIRP Irp;
}COMPLETION_CONTEXT, *PCOMPLETION_CONTEXT;

/* Custom Parameters */
typedef struct _CUSTOMPARAM
{
    PIRP SrbIrp;

	NTSTATUS IoctlStatus;
	PIRP IoctlIrp;
	KEVENT IoctlEvent;

	NTSTATUS BulkStatus;
	PIRP BulkIrp;
	KEVENT BulkEvent;
	ULONG BulkTransferLength;
	BOOLEAN	Read;

	UCHAR SeqBuffer[128];
	UCHAR CdbBuffer[16];

	ULONG BytesTransferred;

	UCHAR CylHiLow[2];
	BOOLEAN HiLowValid;

	COMPLETION_CONTEXT ioctlContext;
	COMPLETION_CONTEXT bulkContext;

} CUSTOMPARAM, *PCUSTOMPARAM;

typedef struct _ATACOMMANDREGS
{
    UCHAR   regFlags;
    UCHAR   cmdFlags;

    UCHAR   features;
    UCHAR   sectorCount;
    UCHAR   sectorNum;
    USHORT  cylinder;
    UCHAR   head;
    UCHAR   cmd;
} ATACOMMANDREGS, *PATACOMMANDREGS;

typedef struct _URB_BULK_OR_INTERRUPT_TRANSFER *PURB_BULK_OR_INTERRUPT_TRANSFER;