/***************************************************************************
 * USBIOCTL.C - USB Mass Storage Driver 
 *
 * This source file contains the dispatch routines which handle:
 *
 * IRP_MJ_DEVICE_CONTROL
 * IRP_MJ_SCSI
 *
 *
 * Copyright (C) 1998-2000 In-System Design, Inc.                                
 */                                                                         

/* includes */
#include "isdcust.h"


/**************************************************************************
 * USBMASS_725Init
 *                                                                         
 * Initialize the 725 device.
 *
 * RETURNS:
 *    TRUE if successful, FALSE otherwise.
 */                                                                        
BOOLEAN
USBMASS_725Init
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
    NTSTATUS                ntStatus;
	PCUSTOMPARAM 				pFs;
	BOOLEAN success = FALSE;
	UCHAR	seqLength;
	static  UCHAR seqInitFalstaffLite[]={
			0x00,0x00,SEQ_BLOCKING_BIT, 			   // Loop Count (WORD) & Block bit (bit 8 of BYTE)
	        (SEQ_RRMW|REG_CTRL),0x04,0x0C,             // Put falstaff lite in Status mode
	        (SEQ_NOOP|0x0F),0xFF, 					   // wait for a while before check status
		    (SEQ_RRCM|REG_STAT),0xFF,0xFF, 0x18, 0x18, // Wait for status to report Device Opterational
	        (SEQ_RRMW|REG_CTRL),0x0C,0x0C,             // Put falstaff lite device in Run/Charge mode
	};
	static  UCHAR seqInitFalstaff[]={
			0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
			(SEQ_RRMW|REG_CTRL),0x00,0x04, 			// if using nInit as reset, set RESET- bit high
	        (SEQ_NOOP|0x0F),0xFF, 					// wait for reset to take affect
			(SEQ_RRMW|REG_CTRL),0x04,0x04, 			// if using nInit as reset, set RESET- bit low
	};
	static  UCHAR seqInituDrive[]={
			0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
	        (SEQ_RRMW|REG_ATAA),0x1F,0x1F,          // Ensure CS1, CS0, DA2, DA1, & DA0 are de-asserted
	        (SEQ_RRMW|REG_CTRL),0x00,0x04,          // Put uDrive device into power on mode
	};
	UCHAR seqInitDevice[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_CCTR),0x40,0x40, 			// Set Slow RRCM bit
		(SEQ_RRMW|REG_CTRL),0x80,0x80, 			// Set RESET- bit low
		(SEQ_RRMW|REG_ECRR),0xE0,0xE0, 			// Set into ATA Mode
		(SEQ_RRMW|REG_CCTR),0x80,0x80, 			// Set Reverse Bit 
		(SEQ_RRMW|REG_SETU),REG_SETU_FILTER_SFT_OVERRIDE,REG_SETU_FILTER_SFT_OVERRIDE, // Disable digital filtering
		(SEQ_RWRI|REG_TIMO),0x00, 				// Set to Timeout Register to 0
		(SEQ_RRMW|REG_CTRL),0x00,0x20, 			// Set Direction bit to 0 (Output Mode)
		(SEQ_RRMW|REG_SETU),0x80,0x80, 			// Set Hold Bulk bit to 1 (don't allow bulk transfer)
        (SEQ_NOOP|0x04),0xAA, 					// wait for a while before (approx. 20ms)
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_725Init\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

	USBSendSoftReset(DeviceObject);
	USBWriteRegister(DeviceObject, REG_CCTR, 0x58);

	if (fdoDeviceExtension->DriverFlags & DF_CUST_FALSTAFFLITE)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected Falstaff-Lite device\n"));
		seqLength = sizeof(seqInitFalstaffLite);
	    RtlCopyMemory(pFs->SeqBuffer,
	                  seqInitFalstaffLite,
	                  seqLength);
		}
	else
		{
		if (fdoDeviceExtension->DriverFlags & DF_CUST_UDRIVE)
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected uDrive device\n"));
			seqLength = sizeof(seqInituDrive);
		    RtlCopyMemory(pFs->SeqBuffer,
		                  seqInituDrive,
		                  seqLength);
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected Standard Falstaff device\n"));
			seqLength = sizeof(seqInitFalstaff);
		    RtlCopyMemory(pFs->SeqBuffer,
		                  seqInitFalstaff,
		                  seqLength);
			}
		}

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					seqLength,
					SEQ_TIMEOUT
					);

	if (NT_SUCCESS(ntStatus))
		{
		seqLength = sizeof(seqInitDevice);
	    RtlCopyMemory(pFs->SeqBuffer,
	                  seqInitDevice,
	                  seqLength);

		ntStatus = USBSendSeqString
						(
						DeviceObject,
						pFs->SeqBuffer,
						seqLength,
						SEQ_TIMEOUT
						);

		if (ntStatus != STATUS_SUCCESS)
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Sequence seqInitDevice failed\n"));
			success = FALSE;
			}
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Falstaff/Falstaff-Lite Sequence failed\n"));
		success = FALSE;
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_725Init - %d\n", success));
    return success;
	}


/**************************************************************************
 * USBMASS_SetATAReg
 *                                                                         
 * Set a specific ATA Register
 *
 * RETURNS:
 *    TRUE if successful, FALSE otherwise.
 */                                                                        
NTSTATUS
USBMASS_SetATAReg
	(
    PDEVICE_OBJECT  DeviceObject,
	UCHAR			ataAddress,
	UCHAR			ataValue
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus;
	PCUSTOMPARAM 				pFs;
	UCHAR seqSetATAReg[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RWRI|REG_ATAA), ataAddress, 		
		(SEQ_RWRI|REG_EPPA), ataValue, 			
		0x00
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_SetATAReg\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

    RtlCopyMemory(pFs->SeqBuffer,
                  seqSetATAReg,
                  sizeof(seqSetATAReg)-1);

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqSetATAReg)-1,
					SEQ_TIMEOUT
					);

	if (ntStatus != STATUS_SUCCESS)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Seq Request seqSetATAReg failed\n"));
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_SetATAReg - 0x%x\n", ntStatus));
    return ntStatus;
	}


NTSTATUS
USBMASS_ResetRegsForSeq
	(
    PDEVICE_OBJECT  DeviceObject,
	BOOLEAN resetPipes
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
    NTSTATUS                ntStatus;
	PCUSTOMPARAM 				pFs;
	UCHAR seqResetRegs[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_SETU), REG_SETU_CLEAR_BULK_OUT, REG_SETU_CLEAR_BULK_OUT,
		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,
		(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
		};

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_ResetRegsForSeq\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

    RtlCopyMemory(pFs->SeqBuffer,
                  seqResetRegs,
                  sizeof(seqResetRegs));

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqResetRegs),
					SEQ_TIMEOUT
					);

	if (ntStatus != STATUS_SUCCESS)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Seq Request seqResetRegs failed\n"));
		}
	else
		{
		if (resetPipes)
			{
		    /* Reset the Bulk Endpoint.  This clears the endpoint halt on the   */
		    /* host side, resets the host side data toggle to Data0, and issues */
		    /* the Clear_Feature Endpoint_Stall request to the device.          */
		    ntStatus = USBMASS_ResetPipe(pdoDeviceExtension->ParentFDO,
		                                 fdoDeviceExtension->BulkInPipe->PipeHandle);

		    ntStatus = USBMASS_ResetPipe(pdoDeviceExtension->ParentFDO,
		                                 fdoDeviceExtension->BulkOutPipe->PipeHandle);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_ResetRegsForSeq - 0x%x\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_GetATAReg
 *                                                                         
 * Get a specific ATA Register
 *
 * RETURNS:
 *    TRUE if successful, FALSE otherwise.
 */                                                                        
NTSTATUS
USBMASS_GetATAReg
	(
    PDEVICE_OBJECT  DeviceObject,
	UCHAR			ataAddress,
	PUCHAR 			ataValue
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus, wStatus;
	PCUSTOMPARAM 				pFs;
    PVOID eventArray[2];
    LARGE_INTEGER Timeout;
	UCHAR seqGetATAReg[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
		(SEQ_RWRI|REG_ATAA), ataAddress, 		
		(SEQ_DATI|REG_EPPA), 			
		(SEQ_RRMW|REG_CCTR), 0x00, REG_CCTR_MANUAL_REVERSE,
		(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,
		0x00
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_GetATAReg\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	pFs->IoctlStatus = pFs->BulkStatus = 0;

    RtlCopyMemory(pFs->SeqBuffer,
                  seqGetATAReg,
                  sizeof(seqGetATAReg)-1);

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqGetATAReg)-1,
					0
					);

	if (ntStatus == STATUS_PENDING)
		{
		ntStatus = USBReadBulk
						(
					    DeviceObject,
						ataValue,
						NULL,
						1,
						0
					    );
		if (ntStatus != STATUS_PENDING)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status Bulk Failed - 0x%x\n", ntStatus));
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   seqGetATAReg Failed - 0x%x\n", ntStatus));
		}

	/* check for status not pending */
	if (ntStatus != STATUS_PENDING)
		{
		/* cancel command IRP */
	    if (pFs->IoctlIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Ioctl IRP\n"));
	    	IoCancelIrp(pFs->IoctlIrp);
			}

		/* cancel data IRP */
	    if (pFs->BulkIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
		    IoCancelIrp(pFs->BulkIrp);
			}
		}

	/* setup array of multiple objects */
    eventArray[0] = &pFs->IoctlEvent;
    eventArray[1] = &pFs->BulkEvent;

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for Events to complete\n"));
	/* wait for all objects to complete */
    Timeout.QuadPart = -((LONGLONG) 10*1000*SEQ_TIMEOUT); 
    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
    wStatus = KeWaitForMultipleObjects
				(
				2,  
				eventArray, 
				WaitAll,
				Executive,
				KernelMode,
				FALSE,
				&Timeout,
				NULL
				);

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   wStatus = 0x%x\n", wStatus));

	/* check for timeout */
	if (wStatus == STATUS_TIMEOUT)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Timed out... Cancel all IRPs and wait for all IRPs to complete\n"));
		ASSERT(FALSE);

		/* cancel command IRP */
	    if (pFs->IoctlIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Ioctl IRP\n"));
	    	IoCancelIrp(pFs->IoctlIrp);

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for IoctlEvent to complete\n"));
		    KeWaitForSingleObject(&pFs->IoctlEvent,
	                                  Executive,
	                                  KernelMode,
	                                  FALSE,
	                                  NULL);
			}

		/* cancel data IRP */
	    if (pFs->BulkIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
		    IoCancelIrp(pFs->BulkIrp);

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for BulkEvent to complete\n"));
		    KeWaitForSingleObject(&pFs->BulkEvent,
	                                  Executive,
	                                  KernelMode,
	                                  FALSE,
	                                  NULL);
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Events completed\n"));
		}

	/* check status returned from both command and data IRP for success */
	if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
		{
		ntStatus = STATUS_SUCCESS;
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Read ATA Register = 0x%x\n", *ataValue));
		}
	else
		{
		SEQUENCER_STATUS SeqStatus;

		/* check for status from both command and data IRP for timeout */
		if ((pFs->IoctlStatus != STATUS_SUCCESS) || (pFs->BulkStatus == STATUS_CANCELLED))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Timeout error occurred\n"));

			if (pFs->IoctlStatus != STATUS_SUCCESS)
				{
				/* abort sequencer because of error */
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
				USBAbortSeq(DeviceObject);
				}
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unrecoverable error occurred\n"));
			}

		if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos = 0x%x\n", SeqStatus.seqPos));
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqSTatus call failed\n"));
			}

		ntStatus = (pFs->IoctlStatus != STATUS_SUCCESS) ? pFs->IoctlStatus : pFs->BulkStatus;
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_GetATAReg - 0x%x\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_GetAllATARegs
 *                                                                         
 * Get all ATA registers
 *
 * RETURNS:
 *    TRUE if successful, FALSE otherwise.
 */                                                                        
NTSTATUS
USBMASS_GetAllATARegs
	(
    PDEVICE_OBJECT  DeviceObject,
	PUCHAR 			ataValues
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus, wStatus;
	PCUSTOMPARAM 				pFs;
    PVOID eventArray[2];
    LARGE_INTEGER Timeout;
	UCHAR seqGetAllATARegs[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS, 		
		(SEQ_DATI|REG_EPPA), 			
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ERROR, 		
		(SEQ_DATI|REG_EPPA), 			
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW, 		
		(SEQ_DATI|REG_EPPA), 			
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH, 		
		(SEQ_DATI|REG_EPPA), 			
		(SEQ_RRMW|REG_CCTR), 0x00, REG_CCTR_MANUAL_REVERSE,
		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,
		(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
		0x00
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_GetAllATARegs\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

    RtlCopyMemory(pFs->SeqBuffer,
                  seqGetAllATARegs,
                  sizeof(seqGetAllATARegs)-1);

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqGetAllATARegs)-1,
					0
					);

	if (ntStatus == STATUS_PENDING)
		{
		ntStatus = USBReadBulk
						(
					    DeviceObject,
						ataValues,
						NULL,
						4,
						0
					    );
		if (ntStatus != STATUS_PENDING)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status Bulk Failed - 0x%x\n", ntStatus));
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   seqGetAllATARegs Failed - 0x%x\n", ntStatus));
		}

	/* check for status not pending */
	if (ntStatus != STATUS_PENDING)
		{
		/* cancel command IRP */
	    if (pFs->IoctlIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Ioctl IRP\n"));
	    	IoCancelIrp(pFs->IoctlIrp);
			}

		/* cancel data IRP */
	    if (pFs->BulkIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
		    IoCancelIrp(pFs->BulkIrp);
			}
		}

	/* setup array of multiple objects */
    eventArray[0] = &pFs->IoctlEvent;
    eventArray[1] = &pFs->BulkEvent;

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for Events to complete\n"));
	/* wait for all objects to complete */
    Timeout.QuadPart = -((LONGLONG) 10*1000*SEQ_TIMEOUT); 
    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
    wStatus = KeWaitForMultipleObjects
				(
				2,  
				eventArray, 
				WaitAll,
				Executive,
				KernelMode,
				FALSE,
				&Timeout,
				NULL
				);

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   wStatus = 0x%x\n", wStatus));

	/* check for timeout */
	if (wStatus == STATUS_TIMEOUT)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Timed out... Cancel all IRPs and wait for all IRPs to complete\n"));
		ASSERT(FALSE);

		/* cancel command IRP */
	    if (pFs->IoctlIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Ioctl IRP\n"));
	    	IoCancelIrp(pFs->IoctlIrp);

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for IoctlEvent to complete\n"));
		    KeWaitForSingleObject(&pFs->IoctlEvent,
	                                  Executive,
	                                  KernelMode,
	                                  FALSE,
	                                  NULL);
			}

		/* cancel data IRP */
	    if (pFs->BulkIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
		    IoCancelIrp(pFs->BulkIrp);

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for BulkEvent to complete\n"));
		    KeWaitForSingleObject(&pFs->BulkEvent,
	                                  Executive,
	                                  KernelMode,
	                                  FALSE,
	                                  NULL);
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Events completed\n"));
		}

	/* check status returned from both command and data IRP for success */
	if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
		{
		ntStatus = STATUS_SUCCESS;

		if (pFs->BulkTransferLength == 4)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Read ATA Registers = [0x%x, 0x%x, 0x%x, 0x%x]\n", 
		    		ataValues[0],
		    		ataValues[1],
		    		ataValues[2],
		    		ataValues[3]
		    		));
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Read ATA Registers Error, zero bytes recieved\n"));
			ntStatus = STATUS_IO_DEVICE_ERROR;
			}
		}
	else
		{
		SEQUENCER_STATUS SeqStatus;

		/* check for status from both command and data IRP for timeout */
		if ((pFs->IoctlStatus != STATUS_SUCCESS) || (pFs->BulkStatus == STATUS_CANCELLED))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Timeout error occurred\n"));

			if (pFs->IoctlStatus != STATUS_SUCCESS)
				{
				/* abort sequencer because of error */
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
				USBAbortSeq(DeviceObject);
				}
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unrecoverable error occurred\n"));
			}

		if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos = 0x%x\n", SeqStatus.seqPos));
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqSTatus call failed\n"));
			}

		ntStatus = (pFs->IoctlStatus != STATUS_SUCCESS) ? pFs->IoctlStatus : pFs->BulkStatus;
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_GetAllATARegs - 0x%x\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_WaitForNotBusy
 *                                                                         
 * 
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR 
USBMASS_WaitForNotBusy
	(
    PDEVICE_OBJECT  DeviceObject,
	ULONG 			timeout
	) 
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus, wStatus;
	PCUSTOMPARAM 			pFs;
	LARGE_INTEGER			startTimeout;
    UCHAR	 				retState;
	UCHAR seqWaitForBusy[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 07-08 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 09-13 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 14-18 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_DRQ,
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_WaitForNotBusy\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	/* obtain current tick count, used to determine the timeout for detecting device */
    KeQueryTickCount(&startTimeout);

	/* loop until not busy, timeout or error */
	retState = WAIT_STATE_BUSY;
	while(retState == WAIT_STATE_BUSY)
		{
		wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqWaitForBusy, sizeof(seqWaitForBusy), NULL, NULL, 0, 0, SEQ_TIMEOUT);

		if (wStatus == STATUS_TIMEOUT)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Wait for Busy Request timed out\n"));
			retState = WAIT_STATE_TIMEOUT_ERROR;
			}
		else
			{
			/* check status returned from both command and data IRP for success */
			if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Wait for Busy completed successfully\n"));
				retState = WAIT_STATE_SUCCESS;
				}
			else
				{
				SEQUENCER_STATUS SeqStatus;

			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Wait for Busy Request failed\n"));

				if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
					{
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

					/* check to see if sequencer is active */
					if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
						{
						/* abort sequencer */
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
						USBAbortSeq(DeviceObject);
						}

					if (SeqStatus.seqPos == 7)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device still BUSY, resubmit request\n"));
						retState = WAIT_STATE_BUSY;
						}
					else if (SeqStatus.seqPos == 14)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR, return device error\n"));
						retState = WAIT_STATE_DEVICE_ERROR;
						}
					else if (SeqStatus.seqPos == 19)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ detected, return device DRQ\n"));
						retState = WAIT_STATE_DEVICE_DRQ;
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = WAIT_STATE_RESET_ERROR;
						}
					}
				else
					{
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
					retState = WAIT_STATE_RESET_ERROR;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, FALSE);
				}
			}

		if (timeout)
			{
			/* check for timeout on this request */
			if (USBMASS_ElapsedTimeMs(&startTimeout) >= timeout) 
				{
			    USBMASS_KdPrint(DBGLVL_HIGH, ("   timed out!\n"));
				retState = WAIT_STATE_TIMEOUT_ERROR;
				}
			}
		else
			{
		    KIRQL irql;

		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_HIGH, ("   SRB timed out!\n"));
					retState = WAIT_STATE_TIMEOUT_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_WaitForNotBusy - 0x%x\n", retState));
    return retState;
	}


/**************************************************************************
 * USBMASS_WaitForDRQ
 *                                                                         
 * 
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN 
USBMASS_WaitForDRQ
	(
    PDEVICE_OBJECT  DeviceObject,
	ULONG 			timeout
	) 
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	BOOLEAN 				success = FALSE;
	UCHAR 					state;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_WaitForDRQ\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	state = USBMASS_WaitForNotBusy(DeviceObject, timeout);

	if (state == WAIT_STATE_RESET_ERROR)
		{	
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, failure\n"));
		}
	else if (state == WAIT_STATE_DEVICE_ERROR)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected, failure\n"));
		}
	else if (state == WAIT_STATE_DEVICE_DRQ)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ bit detected, success\n"));
		success = TRUE;
		}
	else 
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   No Device DRQ bit detected, failure\n"));
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_WaitForDRQ - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_CheckForError
 *                                                                         
 * 
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN 
USBMASS_CheckForError
	(
    PDEVICE_OBJECT  DeviceObject,
	ULONG 			timeout
	) 
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	BOOLEAN 				success = FALSE;
	UCHAR 					state;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_CheckForError\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	state = USBMASS_WaitForNotBusy(DeviceObject, timeout);

	if (state == WAIT_STATE_RESET_ERROR)
		{	
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, failure\n"));
		}
	else if (state == WAIT_STATE_DEVICE_ERROR)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected, failure\n"));
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   No Device ERROR bit detected, success\n"));
		success = TRUE;
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_CheckForError - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_IssueSeqBulk
 *                                                                       
 * Issue a Control Sequence and Bulk request, wait for each to complete
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_IssueSeqBulk
	(
    PDEVICE_OBJECT  DeviceObject,
	PVOID SeqBuffer,
	ULONG SeqLength,
	PVOID BulkBuffer,
	PMDL  BulkMdl,
	ULONG BulkLength,
	BOOLEAN read,
	ULONG timeout
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus;
	PCUSTOMPARAM 			pFs;
    LARGE_INTEGER 			delay;
    PVOID 					eventArray[2];
	UINT 					numEvents;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_IssueSeqBulk\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;
	numEvents = 0;
	ntStatus = STATUS_SUCCESS;

	if ((BulkLength) && (BulkBuffer || BulkMdl))
		{
		if (read)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Issuing Bulk In for %d bytes\n", BulkLength));
			ntStatus = USBReadBulk(
						    DeviceObject,
							BulkBuffer,
							BulkMdl,
							BulkLength,
							0
						    );
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Issuing Bulk Out for %d bytes\n", BulkLength));
			ntStatus = USBWriteBulk(
						    DeviceObject,
							BulkBuffer,
							BulkMdl,
							BulkLength,
							0
						    );
			}

		if (ntStatus == STATUS_PENDING)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Setting Bulk Event\n"));
		    eventArray[numEvents] = &pFs->BulkEvent;
			numEvents++;
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bulk Failed - 0x%x\n", ntStatus));
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   No bulk data for this request\n"));
		}

	if ((ntStatus == STATUS_PENDING) || (ntStatus == STATUS_SUCCESS))
		{
		if ((SeqBuffer) && (SeqLength))
			{
			/* setup for processing ATAPI request */
		    RtlCopyMemory(
				pFs->SeqBuffer,
		        SeqBuffer,
		        SeqLength
		        );

			ntStatus = USBSendSeqString(
							DeviceObject,
							pFs->SeqBuffer,
							SeqLength,
							0
							);

			if (ntStatus == STATUS_PENDING)
				{
				/* setup array of multiple objects */
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Setting Sequencer Event\n"));
			    eventArray[numEvents] = &pFs->IoctlEvent;
				numEvents++;
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Sequence Failed - 0x%x\n", ntStatus));
				}
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   No Sequencer for this request\n"));

			/* still want to try/check bulk */
			ntStatus = STATUS_PENDING;
			}
		}

	/* check for status not pending */
	if (ntStatus != STATUS_PENDING)
		{
		/* cancel command IRP */
	    if (pFs->IoctlIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Sequencer IRP\n"));
	    	IoCancelIrp(pFs->IoctlIrp);
			}

		/* cancel data IRP */
	    if (pFs->BulkIrp)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
		    IoCancelIrp(pFs->BulkIrp);
			}
		}

	if (numEvents)
		{
	    KIRQL irql;

		if (timeout)
			{
		    LARGE_INTEGER Timeout;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for Events to complete, using internal timeout\n"));
			Timeout.QuadPart = -((LONGLONG) 10*1000*timeout); 
			/* wait for all objects to complete */
		    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
		    ntStatus = KeWaitForMultipleObjects
						(
						numEvents,  
						eventArray, 
						WaitAll,
						Executive,
						KernelMode,
						FALSE,
						&Timeout,
						NULL
						);
			}
		else	
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Waiting for Events to complete, use global timeout counter only\n"));
			/* wait for all objects to complete */
		    ntStatus = KeWaitForMultipleObjects
						(
						numEvents,  
						eventArray, 
						WaitAll,
						Executive,
						KernelMode,
						FALSE,
						NULL,
						NULL
						);

		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
					ntStatus = STATUS_TIMEOUT;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   No events to wait on\n"));
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Events completed, ntStatus = 0x%x\n", ntStatus));

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_IssueSeqBulk - 0x%x\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_IssueSRST
 *                                                                         
 * Set a specific ATA Register
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_IssueSRST
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus;
	PCUSTOMPARAM 			pFs;
    LARGE_INTEGER 			delay;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_IssueSRST\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_CNTRL, ATA_DC_RESET_CONTROLLER);

	/* check to see if this request failed */
    if (!NT_SUCCESS(ntStatus))
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error issuing SRST\n"));
		}
	else
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Issued SRST\n"));

		/* delay 10ms to give the drive a chance to see it */
		delay.QuadPart = -(10000 * 10); 
		KeDelayExecutionThread(KernelMode, FALSE, &delay);

		ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_CNTRL, ATA_DC_REENABLE_CONTROLLER);

		/* check to see if this request failed */
	    if (!NT_SUCCESS(ntStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error taking drive out of reset\n"));
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Drive is out of reset\n"));

			/* delay 50ms to give the drive a chance to recover after SRST */
			delay.QuadPart = -(10000 * 50); 
			KeDelayExecutionThread(KernelMode, FALSE, &delay);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_IssueSRST - 0x%x\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_AtapiSoftReset
 *                                                                         
 * Perform an Atapi Soft Reset on the device
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS 
USBMASS_AtapiSoftReset
	(
    PDEVICE_OBJECT  DeviceObject
	) 
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS ntStatus = STATUS_SUCCESS;
	PCUSTOMPARAM 				pFs;
    LARGE_INTEGER delay;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_AtapiSoftReset\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_HEAD, pdoDeviceExtension->DeviceHead);

	/* check to see if this request failed */
    if (!NT_SUCCESS(ntStatus))
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error setting Device/Head register\n"));
		}
	else
		{
		ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_COMMAND, ATA_COMMAND_ATAPI_RESET);

		/* check to see if this request failed */
	    if (!NT_SUCCESS(ntStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error issuing Atapi Soft Reset\n"));
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Issued Atapi Soft Reset\n"));

			/* wait for !BSY (at least for a bit) */
			USBMASS_WaitForNotBusy(DeviceObject, USBMASS_ENUM_BSY_TIMEOUT);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_AtapiSoftReset %08X\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * USBMASS_PerformReset
 *                                                                         
 * Perform an entire reset sequence on the device
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS 
USBMASS_PerformReset
	(
    PDEVICE_OBJECT  DeviceObject
	) 
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS ntStatus = STATUS_SUCCESS;
	PCUSTOMPARAM 				pFs;
    LARGE_INTEGER delay;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_PerformReset\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	if (NT_SUCCESS(ntStatus = USBMASS_ResetDevice(DeviceObject)))
		{
		if (NT_SUCCESS(ntStatus = USBMASS_725Init(DeviceObject)))
			{
			/* wait for !BSY (at least for a bit) */
			USBMASS_WaitForNotBusy(DeviceObject, 3000);

			if (NT_SUCCESS(ntStatus = USBMASS_IssueSRST(DeviceObject)))
				{
				if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_ATA_DEVICE))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA reset completed successfully\n"));
					}
				else
					{
					if (NT_SUCCESS(ntStatus = USBMASS_AtapiSoftReset(DeviceObject)))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI reset completed successfully\n"));
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed AtapiSoftReset routine\n"));
						}
					}
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed IssueSRST routine\n"));
				}
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed 725Init routine\n"));
			}
		}

	if (!NT_SUCCESS(ntStatus))
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed to reset device, setup as if the device is disconnected\n"));
        SET_FLAG(pdoDeviceExtension->DeviceFlags, DF_DEVICE_DISCONNECTED);
        
        USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Set Disconnected Flag\n"));
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_PerformReset %08X\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * USBMASS_Identify
 *                                                                         
 * Attempt to process an ATA/ATAPI Identify request.
 *
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS 
USBMASS_Identify
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	BOOLEAN				isAtapi
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension = DeviceObject->DeviceExtension;
	PCUSTOMPARAM 			pFs;
	NTSTATUS				ntStatus;
    UCHAR seqIdentifyCmdIssue[]={
	    0x00,0x00,SEQ_BLOCKING_BIT,
	    (SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DEV_HEAD,
	    (SEQ_RWRI|REG_EPPA), ATA_ADDRESS_DEVHEAD_STD | pdoDeviceExtension->DeviceHead,
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
	    (SEQ_RWRI|REG_ATAA), ATA_ADDRESS_COMMAND,
        (SEQ_RWRI|REG_EPPA), isAtapi ? ATAPI_COMMAND_IDENTIFY : ATA_COMMAND_IDENTIFY,
    };
	UCHAR seqIdentifyMoveData[]={
		0x00,0x00,0x00,
        (SEQ_RRMW|REG_CCTR), 0x80,0x80,
		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
		(SEQ_RRMW|REG_SETU), 0x00,0x80,
		(SEQ_RRMW|REG_CTRL), 0x20,0x20,
		(SEQ_WBIB|0x00), (UCHAR)((dataTransferLength-1)>>8), (UCHAR)dataTransferLength-1,
		(SEQ_RRMW|REG_CTRL), 0x00,0x20,
		(SEQ_RRMW|REG_SETU), 0x80,0x80,
	};

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Identify\n"));
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Setup Identify Command\n"));
    RtlCopyMemory(pFs->SeqBuffer,
                  seqIdentifyCmdIssue,
                  sizeof(seqIdentifyCmdIssue));

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqIdentifyCmdIssue),
					SEQ_TIMEOUT
					);

	if (ntStatus != STATUS_SUCCESS)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Seq Request seqIdentifyCmdIssue failed\n"));
		}
	else
		{
		if (!USBMASS_WaitForDRQ(DeviceObject, USBMASS_ENUM_BSY_TIMEOUT))
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Wait for DRQ failed\n"));
			ntStatus = STATUS_UNSUCCESSFUL;
			}
		else
			{
			USBMASS_IssueSeqBulk(DeviceObject, seqIdentifyMoveData, sizeof(seqIdentifyMoveData), dataBuffer, Mdl, dataTransferLength, TRUE, 0);

			/* check status returned from both command and data IRP for success */
			if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identify data xfer success\n"));

				if (!USBMASS_CheckForError(DeviceObject, USBMASS_ENUM_BSY_TIMEOUT))
					{
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Check for Error failed\n"));
					ntStatus = STATUS_UNSUCCESSFUL;
					}
				else
					{
					ntStatus = STATUS_SUCCESS;
					}
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identify data xfer failed\n"));
				ntStatus = STATUS_UNSUCCESSFUL;
				}
			}
		}


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Identify - 0x%x\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * ATAPI_ProcessNDRequest
 *                                                                         
 * Attempt to process the entire ATAPI No-Data request.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessNDRequest
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID 				cdbBuffer,
	USHORT				cdbLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension = DeviceObject->DeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_COMMAND_COMPLETE;
   	UCHAR seqAtapiNDRequest[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-05 */     (SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, // set direction out for CDB xfer
/* 06-07 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 08-12 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 13-14 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DEV_HEAD,
/* 15-16 */		(SEQ_RWRI|REG_EPPA), ATA_ADDRESS_DEVHEAD_STD | pdoDeviceExtension->DeviceHead,
/* 17-18 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 19-23 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 24-25 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_FEATURES,
/* 26-27 */		(SEQ_RWRI|REG_EPPA), 0x00,
/* 28-29 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_SECT_CNT,
/* 30-31 */		(SEQ_RWRI|REG_EPPA), 0x00,
/* 32-33 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 34-35 */		(SEQ_RWRI|REG_EPPA), 0xFF,
/* 36-37 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 38-39 */		(SEQ_RWRI|REG_EPPA), 0xFE,
/* 40-41 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_COMMAND,
/* 42-43 */		(SEQ_RWRI|REG_EPPA), ATAPI_COMMAND_PACKET,
/* 44-45 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 46-50 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 51-52 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 53-57 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 58-60 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 61-62 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 63-65 */		(SEQ_WBOB|0x00),     0x00, 0x0B,
/* 66-68 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 69-70 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 71-75 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 76-77 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 78-82 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 83-87 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_DRQ,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessNDRequest\n"));
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	/* Send the CDB (all bytes should fit in 725 buffer) */
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Issuing CDB Bulk Out\n"));
	ntStatus = USBMASS_IssueSeqBulk(DeviceObject, NULL, 0, cdbBuffer, NULL, cdbLength, FALSE, 0);
	if (ntStatus != STATUS_SUCCESS)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   CDB Bulk Out failed - 0x%x\n", ntStatus));
		retState = ATAPI_STATE_RESET_ERROR;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessNDRequest - 0x%x\n", retState));
		return retState;
		}

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqAtapiNDRequest, sizeof(seqAtapiNDRequest), NULL, NULL, 0, 0, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if (pFs->IoctlStatus == STATUS_SUCCESS)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI No-Data Request completed successfully\n"));
			retState = ATAPI_STATE_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI No-Data Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{
					case 13:
					case 24:
					case 51:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before CDB, resubmit request\n"));
						retState = ATAPI_STATE_DEVICE_BUSY;
						break;

					case 58:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					case 76:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, resubmit data xfer only\n"));
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 83:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after CDB, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 88:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ present after CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessNDRequest - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessNDCDB
 *                                                                         
 * Process the ATAPI No-Data request starting at CDB.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessNDCDB
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_COMMAND_COMPLETE;
   	UCHAR seqAtapiNDCDB[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 07-08 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 09-13 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 14-16 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 17-18 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 19-21 */		(SEQ_WBOB|0x00),     0x00, 0x0B,
/* 22-24 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 25-26 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 27-31 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 32-33 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 34-38 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 39-43 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessNDCDB\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqAtapiNDCDB, sizeof(seqAtapiNDCDB), NULL, NULL, 0, 0, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if (pFs->IoctlStatus == STATUS_SUCCESS) 
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI No-Data Request completed successfully\n"));
			retState = ATAPI_STATE_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI No-Data Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{

					case 07:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before CDB, resubmit request\n"));
						retState = ATAPI_STATE_DEVICE_BUSY;
						break;

					case 14:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					case 32:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, resubmit data xfer only\n"));
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 39:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after CDB, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 44:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ present after CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessNDCDB - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_NDRequest
 *                                                                         
 * Issue the ATAPI No-Data request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
ATAPI_NDRequest
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID 				cdbBuffer,
	USHORT				cdbLength
    )
	{
    PIRP             		Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				processing, success = TRUE;
	UCHAR					curState;
    KIRQL 					irql;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_NDRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

	/* set state to attempt to process the entire ATAPI request */
	curState = ATAPI_STATE_COMMAND_START;

	/* loop until done processing the request */
	processing = TRUE;
	while (processing)
		{
		/* perform function based on current state */
		switch (curState)
			{
			case ATAPI_STATE_COMMAND_START:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_START state\n"));
				curState = ATAPI_ProcessNDRequest(DeviceObject, cdbBuffer, cdbLength);
				break;

			case ATAPI_STATE_DEVICE_BUSY:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_BUSY state\n"));
				curState = ATAPI_ProcessNDCDB(DeviceObject);
				break;

			case ATAPI_STATE_COMPLETE_TIMEOUT:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMPLETE_TIMEOUT state\n"));

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				{
				UCHAR state;

				state = USBMASS_WaitForNotBusy(DeviceObject, 0);

				if (state == WAIT_STATE_RESET_ERROR)
					{	
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
				else if (state == WAIT_STATE_DEVICE_ERROR)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
					curState = ATAPI_STATE_DEVICE_ERROR;
					}
				}
				break;

			case ATAPI_STATE_RESET_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_RESET_ERROR state\n"));

				if (!NT_SUCCESS(USBMASS_PerformReset(DeviceObject)))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed PerformReset routine\n"));
					ASSERT(FALSE);
					}

				/* set error status */
		        srb->SrbStatus = SRB_STATUS_BUS_RESET;

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_DEVICE_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_ERROR state\n"));

				/* set error state */
		        srb->SrbStatus = SRB_STATUS_ERROR;

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_COMMAND_COMPLETE:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_COMPLETE state\n"));
				processing = FALSE;
				break;

			default:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown State 0x%x, reset device and end command\n"));
				ASSERT(FALSE);
				curState = ATAPI_STATE_RESET_ERROR;
				break;
			}


		if ((processing) && (curState != ATAPI_STATE_COMMAND_COMPLETE))
			{
			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_NDRequest - %d\n", success));
	return success;
 	}


/**************************************************************************
 * ATAPI_StartDataRequest
 *                                                                         
 * Attempt to setup and process the CDB part of the ATAPI Data request.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_StartDataRequest
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID 				cdbBuffer,
	USHORT				cdbLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension = DeviceObject->DeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_CYLHILO;
   	UCHAR seqAtapiStart[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-05 */     (SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, // set direction out for CDB xfer
/* 06-07 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 08-12 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 13-14 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DEV_HEAD,
/* 15-16 */		(SEQ_RWRI|REG_EPPA), ATA_ADDRESS_DEVHEAD_STD | pdoDeviceExtension->DeviceHead,
/* 17-18 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 19-23 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 24-25 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_FEATURES,
/* 26-27 */		(SEQ_RWRI|REG_EPPA), 0x00,
/* 28-29 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_SECT_CNT,
/* 30-31 */		(SEQ_RWRI|REG_EPPA), 0x00,
/* 32-33 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 34-35 */		(SEQ_RWRI|REG_EPPA), 0xFF, // (SEQ_RWRI|REG_EPPA), (UCHAR)(cylValue >> 8),
/* 36-37 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 38-39 */		(SEQ_RWRI|REG_EPPA), 0xFE, // (SEQ_RWRI|REG_EPPA), (UCHAR)(cylValue),
/* 40-41 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_COMMAND,
/* 42-43 */		(SEQ_RWRI|REG_EPPA), ATAPI_COMMAND_PACKET,
/* 44-45 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 46-50 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 51-52 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 53-57 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 58-60 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 61-62 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 63-65 */		(SEQ_WBOB|0x00),     0x00, 0x0B,
/* 66-68 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 69-70 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 71-75 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 76-77 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 78-82 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 83-87 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 88-90 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 91-93 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 94-95 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 96 */		(SEQ_DATI|REG_EPPA),
/* 97-98 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 99 */		(SEQ_DATI|REG_EPPA),
/* 100-102 */	(SEQ_RRMW|REG_CCTR), 0x00, REG_CCTR_MANUAL_REVERSE,
/* 103-105 */	(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 106-108 */ 	(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 109-111 */	(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_StartDataRequest\n"));
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	/* Send the CDB (all bytes should fit in 725 buffer) */
	ntStatus = USBMASS_IssueSeqBulk(DeviceObject, NULL, 0, cdbBuffer, NULL, cdbLength, FALSE, 0);
	if (ntStatus != STATUS_SUCCESS)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   CDB Bulk Out failed - 0x%x\n", ntStatus));
		retState = ATAPI_STATE_RESET_ERROR;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_StartDataRequest - 0x%x\n", retState));
		return retState;
		}

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqAtapiStart, sizeof(seqAtapiStart), &pFs->CylHiLow[0], NULL, 2, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Start of ATAPI Data Request completed successfully\n"));

			pFs->HiLowValid = TRUE;
			retState = ATAPI_STATE_CHECK_CYLHILO;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{
					case 13:
					case 24:
					case 51:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before CDB, resubmit request\n"));
						retState = ATAPI_STATE_DEVICE_BUSY;
						break;

					case 58:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					case 76:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, check cylinder high/low\n"));
						retState = ATAPI_STATE_CHECK_CYLHILO;
						break;

					case 83:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after CDB, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 88:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present after CDB, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_StartDataRequest - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessDataCDB
 *                                                                         
 * Process the ATAPI Data request starting at CDB.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessDataCDB
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_CYLHILO;
   	UCHAR seqAtapiInCDB[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-04 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 05-09 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 10-11 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 12-16 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 17-19 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 20-21 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 22-24 */		(SEQ_WBOB|0x00),     0x00, 0x0B,
/* 25-27 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 28-29 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 30-34 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 35-36 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 37-41 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 42-46 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,
/* 47-49 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 50-52 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 53-54 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 55 */		(SEQ_DATI|REG_EPPA),
/* 56-57 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 58 */		(SEQ_DATI|REG_EPPA),
/* 59-61 */		(SEQ_RRMW|REG_CCTR), 0x00, REG_CCTR_MANUAL_REVERSE,
/* 62-64 */		(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 65-67 */		(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 68-70 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessDataCDB\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqAtapiInCDB, sizeof(seqAtapiInCDB), &pFs->CylHiLow[0], NULL, 2, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI CDB Request completed successfully\n"));

			pFs->HiLowValid = TRUE;
			retState = ATAPI_STATE_CHECK_CYLHILO;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Data CDB Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{

					case 10:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before CDB, resubmit request\n"));
						retState = ATAPI_STATE_DEVICE_BUSY;
						break;

					case 17:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before CDB, reset device\n"));
						retState = ATAPI_STATE_RESET_ERROR;
						break;

					case 35:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, check cylinder high/low\n"));
						retState = ATAPI_STATE_CHECK_CYLHILO;
						break;

					case 42:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after CDB, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 47:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present after CDB, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessDataCDB - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessInDataLoopOptimized
 *                                                                         
 * Process just the data part.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessInDataLoopOptimized
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerLoop
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerLoop);
	UCHAR 					hiLength = (UCHAR)((bytesPerLoop) >> 8);
	UCHAR 					loLength = (UCHAR)(bytesPerLoop);
	UCHAR 					hiXfer = (UCHAR)((bytesPerLoop-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerLoop-1);
   	UCHAR seqInDataLoop[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-04 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 05-09 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 10-11 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 12-16 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 17-21 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 22-24 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 25-27 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 28-29 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 30-32 */		(SEQ_WBIB|0x00),     hiXfer, loXfer,
/* 33-35 */		(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 36-38 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 39-40 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 41-45 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 46-47 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 48-52 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessInDataLoopOptimized\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqInDataLoop, sizeof(seqInDataLoop), dataBuffer, Mdl, dataTransferLength, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);
				switch(SeqStatus.seqPos)
					{
					case 10:
						if (blockCountProcessed)
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
							retState = ATAPI_STATE_COMPLETE_TIMEOUT;
							}
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, retry...\n"));
							retState = ATAPI_STATE_DATA_OPTIMIZED;
							}
						break;

					case 17:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 22:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 46:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 53:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}
				
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per Loop\n", blockCountProcessed, bytesPerLoop));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerLoop);

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessInDataLoopOptimized - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessInDataLoopDRQ
 *                                                                         
 * Process just the data part, looping on DRQ.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessInDataLoopDRQ
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerDRQ
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerDRQ);
	UCHAR 					hiLength = (UCHAR)((bytesPerDRQ) >> 8);
	UCHAR 					loLength = (UCHAR)(bytesPerDRQ);
	UCHAR 					hiXfer = (UCHAR)((bytesPerDRQ-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerDRQ-1);
   	UCHAR seqInDataLoop[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-04 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 05-09 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 10-11 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 12-16 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 17-21 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 22-23 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 24-28 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, hiLength, 0xFF,
/* 29-30 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 31-35 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, loLength, 0xFF,

/* 36-38 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 39-41 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 42-43 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 44-46 */		(SEQ_WBIB|0x00),     hiXfer, loXfer,
/* 47-49 */		(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 50-52 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 53-54 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 55-59 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 60-61 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 62-66 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessInDataLoopDRQ\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqInDataLoop, sizeof(seqInDataLoop), dataBuffer, Mdl, dataTransferLength, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);
				switch(SeqStatus.seqPos)
					{
					case 10:
						if (blockCountProcessed)
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
							retState = ATAPI_STATE_COMPLETE_TIMEOUT;
							}
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, retry...\n"));
							retState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
							}
						break;

					case 17:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 22:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 29:
					case 36:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cylinder High/Low amount error, check cylinder high/low\n"));
						retState = ATAPI_STATE_DO_CYLHILO;
						break;

					case 60:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 67:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}
				
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per DRQ\n", blockCountProcessed, bytesPerDRQ));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerDRQ);

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessInDataLoopDRQ - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessInData
 *                                                                         
 * Process just the data part.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessInData
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	UCHAR 					hiLength = (UCHAR)((dataTransferLength) >> 8);
	UCHAR 					loLength = (UCHAR)(dataTransferLength);
	UCHAR 					hiXfer = (UCHAR)((dataTransferLength-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(dataTransferLength-1);
   	UCHAR seqInData[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-04 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 05-09 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 10-11 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 12-16 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 17-21 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 22-23 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 24-28 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, hiLength, 0xFF,
/* 29-30 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 31-35 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, loLength, 0xFF,

/* 36-38 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 39-41 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 42-43 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 44-46 */		(SEQ_WBIB|0x00),     hiXfer, loXfer,
/* 47-49 */		(SEQ_RRMW|REG_CCTR), 0x00, REG_CCTR_MANUAL_REVERSE,
/* 50-52 */		(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 53-55 */		(SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 56-58 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 59-60 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 61-65 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 66-67 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 68-72 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessInData\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqInData, sizeof(seqInData), dataBuffer, Mdl, dataTransferLength, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{
					case 10:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after CDB, retry...\n"));
						retState = ATAPI_STATE_DATA_NOT_FINISHED;
						break;

					case 17:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 22:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 29:
					case 36:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cylinder High/Low amount error, check cylinder high/low\n"));
						retState = ATAPI_STATE_DO_CYLHILO;
						break;

					case 66:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						pFs->BytesTransferred += dataTransferLength;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 73:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						pFs->BytesTransferred += dataTransferLength;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessInData - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_InRequest
 *                                                                         
 * Issue the ATAPI IN request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
ATAPI_InRequest
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID 				cdbBuffer,
	USHORT				cdbLength,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				processing, success = TRUE;
	UCHAR					curState;
    KIRQL 					irql;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_InRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

	/* set state to attempt to process the entire ATAPI request */
	curState = ATAPI_STATE_COMMAND_START;

	/* reset to false, always invalidate at start */
	pFs->HiLowValid = FALSE;

	/* loop until done processing the request */
	processing = TRUE;
	while (processing)
		{
		/* perform function based on current state */
		switch (curState)
			{
			case ATAPI_STATE_COMMAND_START:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_START state\n"));
				pFs->HiLowValid = FALSE;
 				curState = ATAPI_StartDataRequest(DeviceObject, cdbBuffer, cdbLength);
				break;

			case ATAPI_STATE_DEVICE_BUSY:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_BUSY state\n"));
				pFs->HiLowValid = FALSE;
				curState = ATAPI_ProcessDataCDB(DeviceObject);
				break;

			case ATAPI_STATE_CHECK_CYLHILO:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_CHECK_CYLHILO state\n"));
				{
				USHORT bytesPerDRQ = 0;
				UCHAR allRegs[8];

				if (!pFs->HiLowValid)
					{
					/* read all registers */
					if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
						{
						if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
							{
							pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
							pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
							pFs->HiLowValid = TRUE;
							}
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					}

				/* check to make sure we didn't fail something */	
				if ((pFs->HiLowValid) && (curState == ATAPI_STATE_CHECK_CYLHILO))
					{
					bytesPerDRQ = pFs->CylHiLow[0];
					bytesPerDRQ = bytesPerDRQ << 8;
					bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

					/* Check to make sure bytesPerDRQ is not greater than dataTransferLength, if so */
					/* then this means the device wants to transfer more data than we requested so  */
					/* this would be a 'phase error', in which case we report an error and reset    */
					/* the device.                                                                  */
					if (bytesPerDRQ > (dataTransferLength - pFs->BytesTransferred))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Phase Error - bytesPerDRQ greater than bytes requested, reset device\n"));
						ASSERT(FALSE);
						curState = ATAPI_STATE_RESET_ERROR;
						}
					else
						{
						/* check to determine if we can read the entire data transfer length in one DRQ */
						if (bytesPerDRQ == dataTransferLength) 
							{
							curState = ATAPI_STATE_DATA_NOT_FINISHED;
							}
						else	
							{
							UCHAR cdbOp = srb->Cdb[0];

							if ((cdbOp == SCSIOP_READ || cdbOp == SCSIOP_READ6) && (!(dataTransferLength % 64)))
								{
								curState = ATAPI_STATE_DATA_OPTIMIZED;
								}
							else	
								{
								if (!(((dataTransferLength / bytesPerDRQ) * bytesPerDRQ) % 64))
									{
									curState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
									}
								else
									{
									curState = ATAPI_STATE_DATA_NOT_FINISHED;
									}
								}
							}
						}
					}
				}
				break;

			case ATAPI_STATE_DATA_OPTIMIZED:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_OPTIMIZED state\n"));
				{
				UCHAR cdbOp = srb->Cdb[0];
				ULONG blockCount;

	            if (cdbOp == SCSIOP_READ6)
	            	{
	            	blockCount = (ULONG)(srb->Cdb[2])<<16 | (ULONG)(srb->Cdb[3])<<8 | srb->Cdb[4];
					}
				else
					{
	            	blockCount = (ULONG)(srb->Cdb[7])<<8 | srb->Cdb[8];
					}

				if (blockCount)
					{
					if (!(dataTransferLength % blockCount))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Looping %d times to read %d total bytes\n", blockCount, dataTransferLength));
						pFs->HiLowValid = FALSE;
						curState = ATAPI_ProcessInDataLoopOptimized(DeviceObject, dataBuffer, Mdl, dataTransferLength, (USHORT)(dataTransferLength/blockCount));
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid blockCount, try looping on DRQ\n"));
						curState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   blockCount must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DATA_LOOP_ON_DRQ:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_LOOP_ON_DRQ state\n"));
				{
				USHORT bytesPerDRQ;

				/* set bytes per DRQ */
				bytesPerDRQ = pFs->CylHiLow[0];
				bytesPerDRQ = bytesPerDRQ << 8;
				bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

				if (bytesPerDRQ)
					{
					/* calculate the most we can transfer based on bytePerDRQ */
					ULONG bytesThisXfer = ((dataTransferLength / bytesPerDRQ) * bytesPerDRQ);

				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Looping to transfer %d bytes of %d total bytes\n", bytesThisXfer, dataTransferLength));
					pFs->HiLowValid = FALSE;
					curState = ATAPI_ProcessInDataLoopDRQ(DeviceObject, dataBuffer, Mdl, bytesThisXfer, bytesPerDRQ);

					/* check to see if we completed the transfer, but more data needs to be processed.  For example */
					/* if the dataTransferLength was not on a bytesPerDRQ boundary.                                 */
					if ((curState == ATAPI_STATE_COMMAND_COMPLETE) && ( bytesThisXfer < dataTransferLength))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Need to transfer %d more bytes\n", (dataTransferLength - bytesThisXfer)));
						curState = ATAPI_STATE_DATA_NOT_FINISHED;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DO_CYLHILO:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DO_CYLHILO state\n"));
				{
				UCHAR allRegs[8];

				/* read all registers */
				if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
					{
					if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
						{
						pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
						pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
						pFs->HiLowValid = TRUE;

						if (allRegs[REG_ALT_STATUS] & ATA_STATUS_DRQ)
							{
							curState = ATAPI_STATE_DATA_NOT_FINISHED;
							}
						else
							{
							/* no more data to process */
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ bit not selected, no more data\n"));
							curState = ATAPI_STATE_COMMAND_COMPLETE;
							}
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DATA_NOT_FINISHED:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_NOT_FINISHED state\n"));
				{
				USHORT bytesPerDRQ;

				/* set bytes per DRQ */
				bytesPerDRQ = pFs->CylHiLow[0];
				bytesPerDRQ = bytesPerDRQ << 8;
				bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

				if (bytesPerDRQ)
					{
					if (dataBuffer)
						{
						PVOID buffer;

						buffer = (PUCHAR)dataBuffer + pFs->BytesTransferred;
						pFs->HiLowValid = FALSE;
						curState = ATAPI_ProcessInData(DeviceObject, buffer, NULL, bytesPerDRQ);
						}
					else
						{
					    PMDL  mdl;
					    PVOID mdlVa;

				        mdlVa = (PUCHAR)MmGetMdlVirtualAddress(Mdl) + pFs->BytesTransferred;

			            /* Part of a split request, allocate new partial MDL */
			            mdl = IoAllocateMdl(mdlVa,
			                                bytesPerDRQ,
			                                FALSE,
			                                FALSE,
			                                NULL);
			            if (mdl == NULL)
				            {
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
							curState = ATAPI_STATE_RESET_ERROR;
			    	        }
			            else
			        	    {
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
			                IoBuildPartialMdl(Mdl,
			                                  mdl,
			                                  mdlVa,
			                                  bytesPerDRQ);

							pFs->HiLowValid = FALSE;
							curState = ATAPI_ProcessInData(DeviceObject, NULL, mdl, bytesPerDRQ);

						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
							IoFreeMdl(mdl);
			            	}
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_COMPLETE_TIMEOUT:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMPLETE_TIMEOUT state\n"));

				if (pFs->BytesTransferred < dataTransferLength)
					{
					curState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
					}
				else	
					{
					UCHAR state;

					curState = ATAPI_STATE_COMMAND_COMPLETE;

					state = USBMASS_WaitForNotBusy(DeviceObject, 0);

					if (state == WAIT_STATE_RESET_ERROR)
						{	
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					else if (state == WAIT_STATE_DEVICE_ERROR)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
						curState = ATAPI_STATE_DEVICE_ERROR;
						}
					else if (state == WAIT_STATE_DEVICE_DRQ)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ bit detected\n"));
						curState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
						}
					}
				break;

			case ATAPI_STATE_CHECK_COMMAND_COMPLETE:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_CHECK_COMMAND_COMPLETE state\n"));

				if (pFs->BytesTransferred < dataTransferLength)
					{
					UCHAR allRegs[8];

					/* read all registers */
					if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
						{
						if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
							{
							if (allRegs[REG_ALT_STATUS] & ATA_STATUS_ERROR)
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
								curState = ATAPI_STATE_DEVICE_ERROR;
								}
							else if (allRegs[REG_ALT_STATUS] & ATA_STATUS_DRQ)
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ bit detected\n"));

								pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
								pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
								pFs->HiLowValid = TRUE;

								curState = ATAPI_STATE_DATA_NOT_FINISHED;
								}
							else
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ bit not selected, no more data\n"));
								curState = ATAPI_STATE_COMMAND_COMPLETE;
								}
							}
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed all bytes needed, no need for more data\n"));
					curState = ATAPI_STATE_COMMAND_COMPLETE;
					}
				break;

			case ATAPI_STATE_DEVICE_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_ERROR state\n"));

				/* set error state */
		        srb->SrbStatus = SRB_STATUS_ERROR;

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_RESET_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_RESET_ERROR state\n"));

				pFs->HiLowValid = FALSE;

				if (!NT_SUCCESS(USBMASS_PerformReset(DeviceObject)))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed PerformReset routine\n"));
					ASSERT(FALSE);
					}

				/* set error status */
		        srb->SrbStatus = SRB_STATUS_BUS_RESET;
				ASSERT(FALSE);

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_COMMAND_COMPLETE:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_COMPLETE state\n"));
				processing = FALSE;
				break;

			default:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown State 0x%x, reset device and end command\n"));
				ASSERT(FALSE);
				curState = ATAPI_STATE_RESET_ERROR;
				break;
			}

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d of %d total bytes\n", pFs->BytesTransferred, dataTransferLength));


		if ((processing) && (curState != ATAPI_STATE_COMMAND_COMPLETE))
			{
			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_InRequest - %d\n", success));
	return success;
	}


/**************************************************************************
 * ATAPI_ProcessOutDataLoopOptimized
 *                                                                         
 * Process just the data part, looping on bytesPerLoop.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessOutDataLoopOptimized
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerLoop
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerLoop);
	UCHAR 					hiLength = (UCHAR)((bytesPerLoop) >> 8);
	UCHAR 					loLength = (UCHAR)(bytesPerLoop);
	UCHAR 					hiXfer = (UCHAR)((bytesPerLoop-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerLoop-1);
   	UCHAR seqOutDataLoop[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 07-08 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 09-13 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 14-18 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 19-21 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 22-23 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 24-26 */		(SEQ_WBOB|0x00),     hiXfer, loXfer,
/* 27-29 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 30-31 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 32-36 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 37-38 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 39-43 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessOutDataLoopOptimized\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqOutDataLoop, sizeof(seqOutDataLoop), dataBuffer, Mdl, dataTransferLength, FALSE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Data Optimized Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);

				switch(SeqStatus.seqPos)
					{
					case 07:
						if (blockCountProcessed)
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
							retState = ATAPI_STATE_COMPLETE_TIMEOUT;
							}
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before data xfer, resubmit data xfer\n"));
							retState = ATAPI_STATE_DATA_OPTIMIZED;
							}
						break;

					case 14:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 19:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 37:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 44:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per DRQ\n", blockCountProcessed, bytesPerLoop));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerLoop);

				/* clear Bulk-Out buffer */
				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessOutDataLoopOptimized - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessOutDataLoopDRQ
 *                                                                         
 * Process just the data part, looping on DRQ.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessOutDataLoopDRQ
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerDRQ
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerDRQ);
	UCHAR 					hiLength = (UCHAR)((bytesPerDRQ) >> 8);
	UCHAR 					loLength = (UCHAR)(bytesPerDRQ);
	UCHAR 					hiXfer = (UCHAR)((bytesPerDRQ-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerDRQ-1);
   	UCHAR seqOutDataLoop[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 07-08 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 09-13 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 14-18 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 19-20 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 21-25 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, hiLength, 0xFF,
/* 26-27 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 28-32 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, loLength, 0xFF,

/* 33-35 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 36-37 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 38-40 */		(SEQ_WBOB|0x00),     hiXfer, loXfer,
/* 41-43 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 44-45 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 46-50 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 51-52 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 53-57 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessOutDataLoopDRQ\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqOutDataLoop, sizeof(seqOutDataLoop), dataBuffer, Mdl, dataTransferLength, FALSE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Data DRQ Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);

				switch(SeqStatus.seqPos)
					{
					case 07:
						if (blockCountProcessed)
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
							retState = ATAPI_STATE_COMPLETE_TIMEOUT;
							}
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before data xfer, resubmit data xfer\n"));
							retState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
							}
						break;

					case 14:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 19:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 26:
					case 33:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cylinder High/Low amount error, resubmit using looping data xfer\n"));
						retState = ATAPI_STATE_DO_CYLHILO;
						break;

					case 51:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 58:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per DRQ\n", blockCountProcessed, bytesPerDRQ));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerDRQ);

				/* clear Bulk-Out buffer */
				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessOutDataLoopDRQ - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_ProcessOutData
 *                                                                         
 * Process just the data part.
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATAPI_ProcessOutData
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
	NTSTATUS				ntStatus, wStatus;
    UCHAR	 				retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
	UCHAR 					hiLength = (UCHAR)((dataTransferLength) >> 8);
	UCHAR 					loLength = (UCHAR)(dataTransferLength);
	UCHAR 					hiXfer = (UCHAR)((dataTransferLength-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(dataTransferLength-1);
   	UCHAR seqOutData[] = 
   		{
                0x00,0x00,SEQ_BLOCKING_BIT,
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 07-08 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 09-13 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
/* 14-18 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 19-20 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_HIGH,
/* 21-25 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, hiLength, 0xFF,
/* 26-27 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_CYL_LOW,
/* 28-32 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, loLength, 0xFF,

/* 33-35 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 36-37 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 38-40 */		(SEQ_WBOB|0x00),     hiXfer, loXfer,
/* 41-43 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 44-45 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 46-50 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 51-52 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 53-57 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_ProcessOutData\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqOutData, sizeof(seqOutData), dataBuffer, Mdl, dataTransferLength, FALSE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATAPI_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Data Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				switch(SeqStatus.seqPos)
					{
					case 07:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before data xfer, resubmit data xfer\n"));
						retState = ATAPI_STATE_DATA_NOT_FINISHED;
						break;

					case 14:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR before data xfer, return device error\n"));
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					case 19:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATAPI_STATE_COMMAND_COMPLETE;
						break;

					case 26:
					case 33:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cylinder High/Low amount error, resubmit using looping data xfer\n"));
						retState = ATAPI_STATE_DO_CYLHILO;
						break;

					case 51:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						pFs->BytesTransferred += dataTransferLength;
						retState = ATAPI_STATE_COMPLETE_TIMEOUT;
						break;

					case 58:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						pFs->BytesTransferred += dataTransferLength;
						retState = ATAPI_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATAPI_STATE_RESET_ERROR;
						break;
					}

				/* clear Bulk-Out buffer */
				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATAPI_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_ProcessOutData - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATAPI_OutRequest
 *                                                                         
 * Issue the ATAPI OUT request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
ATAPI_OutRequest
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID 				cdbBuffer,
	USHORT				cdbLength,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				processing, success = TRUE;
	UCHAR					curState;
    KIRQL 					irql;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATAPI_OutRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

	/* set state to attempt to process the entire ATAPI request */
	curState = ATAPI_STATE_COMMAND_START;

	/* loop until done processing the request */
	processing = TRUE;
	while (processing)
		{
		/* perform function based on current state */
		switch (curState)
			{
			case ATAPI_STATE_COMMAND_START:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_START state\n"));
				pFs->HiLowValid = FALSE;
 				curState = ATAPI_StartDataRequest(DeviceObject, cdbBuffer, cdbLength);
				break;

			case ATAPI_STATE_DEVICE_BUSY:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_BUSY state\n"));
				pFs->HiLowValid = FALSE;
				curState = ATAPI_ProcessDataCDB(DeviceObject);
				break;

			case ATAPI_STATE_CHECK_CYLHILO:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_CHECK_CYLHILO state\n"));
				{
				USHORT bytesPerDRQ = 0;
				UCHAR allRegs[8];

				if (!pFs->HiLowValid)
					{
					/* read all registers */
					if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
						{
						if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
							{
							pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
							pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
							pFs->HiLowValid = TRUE;
							}
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					}

				/* check to make sure we didn't fail something */	
				if ((pFs->HiLowValid) && (curState == ATAPI_STATE_CHECK_CYLHILO))
					{
					bytesPerDRQ = pFs->CylHiLow[0];
					bytesPerDRQ = bytesPerDRQ << 8;
					bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

					/* Check to make sure bytesPerDRQ is not greater than dataTransferLength, if so */
					/* then this means the device wants to transfer more data than we requested so  */
					/* this would be a 'phase error', in which case we report an error and reset    */
					/* the device.                                                                  */
					if (bytesPerDRQ > (dataTransferLength - pFs->BytesTransferred))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Phase Error - bytesPerDRQ greater than bytes requested, reset device\n"));
						ASSERT(FALSE);
						curState = ATAPI_STATE_RESET_ERROR;
						}
					else
						{
						/* check to determine if we can read the entire data transfer length in one DRQ */
						if (bytesPerDRQ == dataTransferLength) 
							{
							curState = ATAPI_STATE_DATA_NOT_FINISHED;
							}
						else	
							{
							UCHAR cdbOp = srb->Cdb[0];

							if (cdbOp == SCSIOP_WRITE || cdbOp == SCSIOP_WRITE6 || cdbOp == SCSIOP_WRITECONTINUE)
								{
								curState = ATAPI_STATE_DATA_OPTIMIZED;
								}
							else	
								{
								curState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
								}
							}
						}
					}
				}
				break;

			case ATAPI_STATE_DATA_OPTIMIZED:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_OPTIMIZED state\n"));
				{
				UCHAR cdbOp = srb->Cdb[0];
				ULONG blockCount;

	            if (cdbOp == SCSIOP_WRITE6)
	            	{
	            	blockCount = (ULONG)(srb->Cdb[2])<<16 | (ULONG)(srb->Cdb[3])<<8 | srb->Cdb[4];
					}
	            else if (cdbOp == SCSIOP_WRITE12)
	            	{
	            	blockCount = (ULONG)(srb->Cdb[6])<<24 | (ULONG)(srb->Cdb[7])<<16 | (ULONG)(srb->Cdb[8])<<8 | srb->Cdb[9];
					}
	            else if (cdbOp == SCSIOP_WRITECONTINUE)
	            	{
					/* Write Continue (0xE1) is transfer bytes, not blocks. */
					/* Setting blockCount to 1 ensures that it will transfer the total bytes, not blocks */
	            	blockCount = 1;
					}
				else
					{
	            	blockCount = (ULONG)(srb->Cdb[7])<<8 | srb->Cdb[8];
					}

				if (blockCount)
					{
					if (!(dataTransferLength % blockCount))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Looping %d times to write %d total bytes\n", blockCount, dataTransferLength));
						pFs->HiLowValid = FALSE;
						curState = ATAPI_ProcessOutDataLoopOptimized(DeviceObject, dataBuffer, Mdl, dataTransferLength, (USHORT)(dataTransferLength/blockCount));
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid blockCount, try looping on DRQ\n"));
						curState = ATAPI_STATE_DATA_LOOP_ON_DRQ;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   blockCount must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DATA_LOOP_ON_DRQ:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_LOOP_ON_DRQ state\n"));
				{
				USHORT bytesPerDRQ;

				/* set bytes per DRQ */
				bytesPerDRQ = pFs->CylHiLow[0];
				bytesPerDRQ = bytesPerDRQ << 8;
				bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

				if (bytesPerDRQ)
					{
					/* calculate the most we can transfer based on bytePerDRQ */
					ULONG bytesThisXfer = ((dataTransferLength / bytesPerDRQ) * bytesPerDRQ);

				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Looping to transfer %d bytes of %d total bytes\n", bytesThisXfer, dataTransferLength));
					pFs->HiLowValid = FALSE;
					curState = ATAPI_ProcessOutDataLoopDRQ(DeviceObject, dataBuffer, Mdl, bytesThisXfer, bytesPerDRQ);

					/* check to see if we completed the transfer, but more data needs to be processed.  For example */
					/* if the dataTransferLength was not on a bytesPerDRQ boundary.                                 */
					if ((curState == ATAPI_STATE_COMMAND_COMPLETE) && ( bytesThisXfer < dataTransferLength))
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Need to transfer %d more bytes\n", (dataTransferLength - bytesThisXfer)));
						curState = ATAPI_STATE_DO_CYLHILO;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DO_CYLHILO:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DO_CYLHILO state\n"));
				{
				UCHAR allRegs[8];

				/* read all registers */
				if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
					{
					if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
						{
						pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
						pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
						pFs->HiLowValid = TRUE;

						if (allRegs[REG_ALT_STATUS] & ATA_STATUS_DRQ)
							{
							curState = ATAPI_STATE_DATA_NOT_FINISHED;
							}
						else
							{
							/* no more data to process */
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ bit not selected, no more data\n"));
							curState = ATAPI_STATE_COMMAND_COMPLETE;
							}
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_DATA_NOT_FINISHED:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DATA_NOT_FINISHED state\n"));
				{
				USHORT bytesPerDRQ;

				/* set bytes per DRQ */
				bytesPerDRQ = pFs->CylHiLow[0];
				bytesPerDRQ = bytesPerDRQ << 8;
				bytesPerDRQ = bytesPerDRQ | pFs->CylHiLow[1];
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ = 0x%x\n", bytesPerDRQ));

				if (bytesPerDRQ)
					{
					if (dataBuffer)
						{
						PVOID buffer;

						buffer = (PUCHAR)dataBuffer + pFs->BytesTransferred;
						pFs->HiLowValid = FALSE;
						curState = ATAPI_ProcessOutData(DeviceObject, buffer, NULL, bytesPerDRQ);
						}
					else
						{
					    PMDL  mdl;
					    PVOID mdlVa;

				        mdlVa = (PUCHAR)MmGetMdlVirtualAddress(Mdl) + pFs->BytesTransferred;

			            /* Part of a split request, allocate new partial MDL */
			            mdl = IoAllocateMdl(mdlVa,
			                                bytesPerDRQ,
			                                FALSE,
			                                FALSE,
			                                NULL);
			            if (mdl == NULL)
				            {
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
							curState = ATAPI_STATE_RESET_ERROR;
			    	        }
			            else
			        	    {
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
			                IoBuildPartialMdl(Mdl,
			                                  mdl,
			                                  mdlVa,
			                                  bytesPerDRQ);

							pFs->HiLowValid = FALSE;
							curState = ATAPI_ProcessOutData(DeviceObject, NULL, mdl, bytesPerDRQ);

						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
							IoFreeMdl(mdl);
			            	}
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bytes Per DRQ must be > 0, reset device\n"));
					ASSERT(FALSE);
					curState = ATAPI_STATE_RESET_ERROR;
					}
				}
				break;

			case ATAPI_STATE_COMPLETE_TIMEOUT:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMPLETE_TIMEOUT state\n"));

				if (pFs->BytesTransferred < dataTransferLength)
					{
					curState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
					}
				else	
					{
					UCHAR state;

					curState = ATAPI_STATE_COMMAND_COMPLETE;

					state = USBMASS_WaitForNotBusy(DeviceObject, 0);

					if (state == WAIT_STATE_RESET_ERROR)
						{	
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					else if (state == WAIT_STATE_DEVICE_ERROR)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
						curState = ATAPI_STATE_DEVICE_ERROR;
						}
					else if (state == WAIT_STATE_DEVICE_DRQ)
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ bit detected\n"));
						curState = ATAPI_STATE_CHECK_COMMAND_COMPLETE;
						}
					}
				break;

			case ATAPI_STATE_CHECK_COMMAND_COMPLETE:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_CHECK_COMMAND_COMPLETE state\n"));

				if (pFs->BytesTransferred < dataTransferLength)
					{
					UCHAR allRegs[8];

					/* read all registers */
					if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
						{
						if (!(allRegs[REG_ALT_STATUS] & ATA_STATUS_BUSY))
							{
							if (allRegs[REG_ALT_STATUS] & ATA_STATUS_ERROR)
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
								curState = ATAPI_STATE_DEVICE_ERROR;
								}
							else if (allRegs[REG_ALT_STATUS] & ATA_STATUS_DRQ)
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device DRQ bit detected\n"));

								pFs->CylHiLow[0] = allRegs[REG_CYLINDER_HIGH];
								pFs->CylHiLow[1] = allRegs[REG_CYLINDER_LOW];
								pFs->HiLowValid = TRUE;

								curState = ATAPI_STATE_DATA_NOT_FINISHED;
								}
							else
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ bit not selected, no more data\n"));
								curState = ATAPI_STATE_COMMAND_COMPLETE;
								}
							}
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   BSY bit set, try again\n"));
							}
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
						curState = ATAPI_STATE_RESET_ERROR;
						}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed all bytes needed, no need for more data\n"));
					curState = ATAPI_STATE_COMMAND_COMPLETE;
					}
				break;

			case ATAPI_STATE_DEVICE_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_DEVICE_ERROR state\n"));

				/* set error state */
		        srb->SrbStatus = SRB_STATUS_ERROR;

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_RESET_ERROR:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_RESET_ERROR state\n"));

				pFs->HiLowValid = FALSE;

				if (!NT_SUCCESS(USBMASS_PerformReset(DeviceObject)))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed PerformReset routine\n"));
					ASSERT(FALSE);
					}

				/* set error status */
		        srb->SrbStatus = SRB_STATUS_BUS_RESET;
				ASSERT(FALSE);

				curState = ATAPI_STATE_COMMAND_COMPLETE;
				success = FALSE;
				break;

			case ATAPI_STATE_COMMAND_COMPLETE:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI_STATE_COMMAND_COMPLETE state\n"));
				processing = FALSE;
				break;

			default:
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown State 0x%x, reset device and end command\n"));
				ASSERT(FALSE);
				curState = ATAPI_STATE_RESET_ERROR;
				break;
			}

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d of %d total bytes\n", pFs->BytesTransferred, dataTransferLength));

		if ((processing) && (curState != ATAPI_STATE_COMMAND_COMPLETE))
			{
			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					curState = ATAPI_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATAPI_OutRequest - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_IssueAtapiRequest
 *                                                                         
 * Issue the ATAPI request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_IssueAtapiRequest
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				success = FALSE;
	UCHAR					curState;
    PMDL                    mdl;
    PVOID                   mdlVa;
    PVOID                   transferBuffer;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_IssueAtapiRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

    if ((Irp->MdlAddress != NULL) ||
        ((srb != pdoDeviceExtension->OriginalSrb) && (srb->DataBuffer != NULL)))
		{
		/* setup either transfer buffer or MDL for use */
	    mdl = NULL;
	    transferBuffer = NULL;

	    if (srb == pdoDeviceExtension->OriginalSrb)
		    {
	        /* Check to see if this request is part of a split request. */
	        mdlVa = MmGetMdlVirtualAddress(Irp->MdlAddress);

	        if (mdlVa == (PVOID)srb->DataBuffer)
		        {
	            /* Not part of a split request, use original MDL */
	            mdl = Irp->MdlAddress;
		        }
	        else
	    	    {
	            /* Part of a split request, allocate new partial MDL */
	            mdl = IoAllocateMdl(srb->DataBuffer,
	                                srb->DataTransferLength,
	                                FALSE,
	                                FALSE,
	                                NULL);
	            if (mdl == NULL)
		            {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
	    	        }
	            else
	        	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
	                IoBuildPartialMdl(Irp->MdlAddress,
	                                  mdl,
	                                  srb->DataBuffer,
	                                  srb->DataTransferLength);
	            	}
		        }
	    	}
	    else
		    {
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Using SRB DataBuffer\n"));
	        transferBuffer = srb->DataBuffer;
	    	}

	    if (mdl != NULL || transferBuffer != NULL)
			{
			if (srb->SrbFlags & SRB_FLAGS_DATA_IN)
				{
				success = ATAPI_InRequest(DeviceObject, pFs->CdbBuffer, 12, transferBuffer, mdl, srb->DataTransferLength);
				}
			else 
				{
				success = ATAPI_OutRequest(DeviceObject, pFs->CdbBuffer, 12, transferBuffer, mdl, srb->DataTransferLength);
				}

			/* determine if we need to free an MDL we created */
		    if ((transferBuffer == NULL) && (srb == pdoDeviceExtension->OriginalSrb && mdl != Irp->MdlAddress))
			    {
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
		        IoFreeMdl(mdl);
		    	}
			}
		else
			{
		    Irp->IoStatus.Status = STATUS_UNSUCCESSFUL;
			}
		}
	else
		{
		success = ATAPI_NDRequest(DeviceObject, pFs->CdbBuffer, 12);
		}

	/* if request failed, then determine if Request Sense should be issued */
	if (!success)
		{
		/* check for SRB error */
		if (srb->SrbStatus == SRB_STATUS_ERROR)
			{
	        srb->ScsiStatus = SCSISTAT_CHECK_CONDITION;

			/* determine if we can issue request sense */
	        if (!(srb->SrbFlags & SRB_FLAGS_DISABLE_AUTOSENSE) &&
	            (srb->SenseInfoBufferLength != 0) &&
	            (srb->SenseInfoBuffer != NULL))
				{
				USHORT senseLength = srb->SenseInfoBufferLength;

				/* issue RequestSense */
		        pFs->BytesTransferred = 0;
				RtlZeroMemory(pFs->CdbBuffer, sizeof(pFs->CdbBuffer));
		        pFs->CdbBuffer[0] = SCSIOP_REQUEST_SENSE;
		        pFs->CdbBuffer[4] = (UCHAR)senseLength;
		        if (ATAPI_InRequest(DeviceObject, pFs->CdbBuffer, 12, srb->SenseInfoBuffer, NULL, senseLength))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   RequestSense completed\n"));

		            srb->SrbStatus |= SRB_STATUS_AUTOSENSE_VALID;
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   RequestSense failed\n"));

		            Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR; /* ????? */
					}

		        pFs->BytesTransferred = 0; /* ????? Leave as set? */
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Can't issue RequestSense\n"));

	            Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR; /* ????? */
		        pFs->BytesTransferred = 0; /* ????? Leave as set? */
				}
			}
		else
			{
			if (srb->SrbStatus == SRB_STATUS_BUS_RESET)
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device was Reset\n"));
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Device Error\n"));
				}

            Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR; /* ????? */
	        pFs->BytesTransferred = 0; /* ????? Leave as set? */
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_IssueAtapiRequest - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_Scsi2Atapi
 *                                                                         
 * Does any conversion from SCSI to ATAPI
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_Scsi2Atapi
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				success = TRUE;
	BOOLEAN 				performAtapiConversion = FALSE;
	PVOID					validSrbBuffer = NULL;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Scsi2Atapi\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

	/* setup internal CDB buffer */
	RtlZeroMemory(pFs->CdbBuffer, sizeof(pFs->CdbBuffer));
	RtlCopyMemory(pFs->CdbBuffer, srb->Cdb, srb->CdbLength);

	/* check device type... we know certain devices will need ATAPI conversion because we are */
	/* an ATAPI bridge chip                                                                   */
	if ((pdoDeviceExtension->InquiryData.DeviceType == READ_ONLY_DIRECT_ACCESS_DEVICE) ||
		(pdoDeviceExtension->InquiryData.DeviceType == OPTICAL_DEVICE))
		{
		performAtapiConversion = TRUE;
		}

	/* get an address to the buffer I can work with */
	if (Irp->MdlAddress != NULL)
		{
		validSrbBuffer = MmGetSystemAddressForMdl(Irp->MdlAddress);
		}
	else
		{
		validSrbBuffer = srb->DataBuffer;
		}

	/* if we are doing a conversion, make sure we have a valid buffer address */
	if (performAtapiConversion)
		{
		if (validSrbBuffer)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Using Srb Buffer Address 0x%x for conversion\n", validSrbBuffer));
			}
		else
			{
			performAtapiConversion = FALSE;
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid Buffer Address, no conversion will be performed\n"));
			}
		}

	/* convert any required SCSI request to its ATAPI equiviant */
	if (performAtapiConversion)
		{
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Perform required SCSI to ATAPI conversion\n"));

	    switch (pFs->CdbBuffer[0])
		    {

	        case SCSIOP_MODE_SELECT:
				{
	            PMODE_SELECT_10 modeSelect10 = (PMODE_SELECT_10)pFs->CdbBuffer;
	            UCHAR Length = ((PCDB)pFs->CdbBuffer)->MODE_SELECT.ParameterListLength;
				UCHAR PFBit = ((PCDB)pFs->CdbBuffer)->MODE_SELECT.PFBit;
				UCHAR SPBit = ((PCDB)pFs->CdbBuffer)->MODE_SELECT.SPBit;

	            ZeroMemory(pFs->CdbBuffer, MAXIMUM_CDB_SIZE);

	            modeSelect10->OperationCode = ATAPI_MODE_SELECT;
	            modeSelect10->PFBit = PFBit;
	            modeSelect10->SPBit = SPBit;
	            modeSelect10->ParameterListLengthMsb = 0;
	            modeSelect10->ParameterListLengthLsb = Length;
				}
	            break;

	        case SCSIOP_MODE_SENSE:
				{
	            PMODE_SENSE_10 modeSense10 = (PMODE_SENSE_10)pFs->CdbBuffer;
	            UCHAR PageCode = ((PCDB)pFs->CdbBuffer)->MODE_SENSE.PageCode;
	            UCHAR Pc = ((PCDB)pFs->CdbBuffer)->MODE_SENSE.Pc;
	            UCHAR Length = ((PCDB)pFs->CdbBuffer)->MODE_SENSE.AllocationLength;

				/* Translate 1Ah Mode Sense to 5Ah Mode Sense if buffer size allows and always for MO */
				if ((srb->DataTransferLength >= sizeof(MODE_PARAMETER_HEADER10)) || (pdoDeviceExtension->InquiryData.DeviceType == OPTICAL_DEVICE))
					{
		            ZeroMemory(pFs->CdbBuffer, MAXIMUM_CDB_SIZE);

		            modeSense10->OperationCode = ATAPI_MODE_SENSE;
		            modeSense10->PageCode = PageCode;
		            modeSense10->Pc = Pc;
		            modeSense10->ParameterListLengthMsb = 0;
		            modeSense10->ParameterListLengthLsb = Length;
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid SRB DataTransferLength, no conversion will be done\n"));
					}
				}
	            break;

// this translation causes problems with MO drives and does not seem to be correct.  This
// code is here because it was originally ported from an older code base created by Syzygy
// contract work.  I'll just take it out for right now.
#if 0
	        case SCSIOP_FORMAT_UNIT:
	        	pFs->CdbBuffer[0] = ATAPI_FORMAT_UNIT;
		        break;
#endif
	        default:
	            break;
		    }
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Scsi2Atapi - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_Atapi2Scsi
 *                                                                         
 * Does any conversion from ATAPI to SCSI
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_Atapi2Scsi
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    BOOLEAN 				success = TRUE;
	BOOLEAN 				performAtapiConversion = FALSE;
	PVOID					validSrbBuffer = NULL;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Atapi2Scsi\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

	/* get an address to the buffer I can work with */
	if (Irp->MdlAddress != NULL)
		{
		validSrbBuffer = MmGetSystemAddressForMdl(Irp->MdlAddress);
		}
	else
		{
		validSrbBuffer = srb->DataBuffer;
		}

    cdb = (PCDB)srb->Cdb;

	if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SCSI_PASS_THROUGH))
		{
		/* check device type... we know certain devices will need ATAPI conversion because we are */
		/* an ATAPI bridge chip                                                                   */
		if ((pdoDeviceExtension->InquiryData.DeviceType == READ_ONLY_DIRECT_ACCESS_DEVICE) ||
			(pdoDeviceExtension->InquiryData.DeviceType == OPTICAL_DEVICE))
			{
			performAtapiConversion = TRUE;
			}

		/* if we are doing a conversion, make sure we have a valid buffer address */
		if (performAtapiConversion)
			{
			if (validSrbBuffer)
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Using Srb Buffer Address 0x%x for conversion\n", validSrbBuffer));
				}
			else
				{
				performAtapiConversion = FALSE;
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid Buffer Address, no conversion will be performed\n"));
				}
			}

	    switch (srb->Cdb[0])
		    {
	        case SCSIOP_INQUIRY:
				{
			    PINQUIRYDATA    pInquiry;

				pInquiry = (PINQUIRYDATA)validSrbBuffer;

				if (performAtapiConversion)
					{
					pInquiry->Versions = 0x02;
					}

				if (validSrbBuffer && (fdoDeviceExtension->MountAsRemovable & MR_REMOVABLE))
					{	
					pInquiry->RemovableMedia = 1;
					}
				}
	            break;

			case SCSIOP_MODE_SENSE:			
				{
				/* convert any required SCSI request to its ATAPI equiviant */
				if (performAtapiConversion)
					{
		            PMODE_PARAMETER_HEADER   hdr6;
		            PMODE_PARAMETER_HEADER10 hdr10;

					/* Translate 1Ah Mode Sense to 5Ah Mode Sense if buffer size allows and always for MO */
					if ((srb->DataTransferLength >= sizeof(MODE_PARAMETER_HEADER10)) || (pdoDeviceExtension->InquiryData.DeviceType == OPTICAL_DEVICE))
						{
			            hdr6  = (PMODE_PARAMETER_HEADER)validSrbBuffer;
			            hdr10 = (PMODE_PARAMETER_HEADER10)validSrbBuffer;

			            hdr6->ModeDataLength = hdr10->ModeDataLength[1];  // LSB
			            hdr6->MediumType = hdr10->MediumType;
			            hdr6->DeviceSpecificParameter = hdr10->DeviceSpecificParameter;
			            hdr6->BlockDescriptorLength = hdr10->BlockDescriptorLength[1]; // LSB

			            hdr6++;     // Advance past headers
			            hdr10++;

			            // Copy everything past the 10-byte header
			            //
			            RtlCopyMemory(hdr6,
			                          hdr10,
			                          srb->DataTransferLength - sizeof(*hdr10));

			            // Adjust the return size to account for the smaller header
			            //
			            srb->DataTransferLength   -= (sizeof(*hdr10) - sizeof(*hdr6));
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid SRB DataTransferLength, no conversion will be done\n"));
						}
					}
				}
				break;

	        default:
	            break;
		    }
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Atapi2Scsi - %d\n", success));
	return success;
	}


/**************************************************************************
 * ATA_IssueCmdSeq
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_IssueCmdSeq
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS ataRegs
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState = ATA_STATE_SUCCESS;
	UCHAR					cmdSeq[128];
	PUCHAR					seqPtr;
	UCHAR 					seqBsyBeforeDevSel, seqBsyBeforeCmd, seqBsyAfterCmd, seqErrAfterCmd, seqDrqAfterCmd;
	NTSTATUS				wStatus;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_IssueCmdSeq\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	/* setup sequence for ATA command */
	seqPtr = cmdSeq;

    /* setup sequence header */
    *seqPtr++ = 0x00; *seqPtr++ = 0x00; *seqPtr++ = SEQ_BLOCKING_BIT;

	/* set the manual reverse bit */
	*seqPtr++ = (SEQ_RRMW|REG_CCTR); *seqPtr++ = REG_CCTR_MANUAL_REVERSE; *seqPtr++ = REG_CCTR_MANUAL_REVERSE;

    /* poll STATUS for !BSY */
    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_ALT_STATUS;
    *seqPtr++ = (SEQ_RRCM|REG_EPPA); *seqPtr++ = 0xFF; *seqPtr++ = 0xFF; *seqPtr++ = 0x00; *seqPtr++ = 0x80;
	seqBsyBeforeDevSel = (seqPtr - cmdSeq) - 3; // -3 for header bytes that the chip doesn't receive

	/* setup head register */
    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_DEV_HEAD;
    *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = ATA_ADDRESS_DEVHEAD_STD | pdoDeviceExtension->DeviceHead;

    /* poll STATUS for !BSY */
    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_ALT_STATUS;
    *seqPtr++ = (SEQ_RRCM|REG_EPPA); *seqPtr++ = 0xFF; *seqPtr++ = 0xFF; *seqPtr++ = 0x00; *seqPtr++ = 0x80;
	seqBsyBeforeCmd = (seqPtr - cmdSeq) - 3; // -3 for header bytes that the chip doesn't receive

    /* setup required ATA regs */
    if (ataRegs->regFlags & ATA_REGMASK_FEATURES) 
    	{
        *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_FEATURES;
        *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = ataRegs->features;
		}
    if (ataRegs->regFlags & ATA_REGMASK_SECTOR_COUNT) 
    	{
	    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_SECT_CNT;
	    *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = ataRegs->sectorCount;
	    }
    if (ataRegs->regFlags & ATA_REGMASK_SECTOR_NUM) 
    	{
	    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_SECT_NUM;
	    *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = ataRegs->sectorNum;
	    }
    if (ataRegs->regFlags & ATA_REGMASK_CYLINDER) 
    	{
        *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_CYL_LOW;
        *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = (UCHAR)ataRegs->cylinder;
        *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_CYL_HIGH;
        *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = (UCHAR)(ataRegs->cylinder>>8);
	    }
    if (ataRegs->regFlags & ATA_REGMASK_HEAD) 
    	{
	    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_DEV_HEAD;
	    *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = (ATA_ADDRESS_DEVHEAD_STD | pdoDeviceExtension->DeviceHead | ataRegs->head);
		}
		
    /* setup command register */
    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_COMMAND;
    *seqPtr++ = (SEQ_RWRI|REG_EPPA); *seqPtr++ = (UCHAR)(ataRegs->cmd);

    /* check to determine if to wait for !BSY and check !ERR after cmd */
    if (ataRegs->cmdFlags & ATA_CMDMASK_NO_CHECK_FOR_BSY_AND_ERR_AFTER_CMD)
		{										   
	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Skipping check for !BSY and ERR\n"));

		/* set to some bogus value */
		seqBsyAfterCmd = seqErrAfterCmd = seqDrqAfterCmd = 0xFF;
		}
	else
    	{
		/* add check for !BSY */
	    if (ataRegs->cmdFlags & ATA_CMDMASK_WILL_XFER_DATA) 
	    	{
		    /* set to Alt Status if tranfering data */
		    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_ALT_STATUS;
			}
		else
			{
		    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_STATUS;
			}
	    *seqPtr++ = (SEQ_RRCM|REG_EPPA); *seqPtr++ = 0xFF; *seqPtr++ = 0xFF; *seqPtr++ = 0x00; *seqPtr++ = 0x80;
		seqBsyAfterCmd = (seqPtr - cmdSeq) - 3; // -3 for header bytes that the chip doesn't receive

	    *seqPtr++ = (SEQ_RWRI|REG_ATAA); *seqPtr++ = ATA_ADDRESS_STATUS;

		/* add check for !ERR */
	    *seqPtr++ = (SEQ_RRCM|REG_EPPA); *seqPtr++ = 0x00; *seqPtr++ = 0x00; *seqPtr++ = 0x00; *seqPtr++ = 0x01;
		seqErrAfterCmd = (seqPtr - cmdSeq) - 3; // -3 for header bytes that the chip doesn't receive

	    /* check to determine if to wait for DRQ after cmd */
	    if (ataRegs->cmdFlags & ATA_CMDMASK_CHECK_FOR_DRQ_AFTER_CMD) 
	    	{
	        *seqPtr++ = (SEQ_RRCM|REG_EPPA); *seqPtr++ = 0x00; *seqPtr++ = 0x00; *seqPtr++ = ATA_STATUS_DRQ; *seqPtr++ = ATA_STATUS_DRQ;
	        seqDrqAfterCmd = (seqPtr - cmdSeq) - 3; // -3 for header bytes that the chip doesn't receive
	    	} 
	    else
			{
			/* set to some bogus value */
			seqDrqAfterCmd = 0xFF;
			}
	    } 

	wStatus = USBMASS_IssueSeqBulk(DeviceObject, cmdSeq, (seqPtr - cmdSeq), NULL, NULL, 0, 0, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATA_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if (pFs->IoctlStatus == STATUS_SUCCESS) 
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Command completed successfully\n"));
			retState = ATA_STATE_SUCCESS;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Commnad Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

				if (SeqStatus.seqPos == 0xFF)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid Sequencer state, reset device\n"));
					ASSERT(FALSE);
					retState = ATA_STATE_RESET_ERROR;
					}
				else if (SeqStatus.seqPos == seqBsyBeforeDevSel)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before device select, resubmit request\n"));
					retState = ATA_STATE_RESUBMIT;
					}
				else if (SeqStatus.seqPos == seqBsyBeforeCmd)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY before command, resubmit request\n"));
					retState = ATA_STATE_RESUBMIT;
					}
				else if (SeqStatus.seqPos == seqBsyAfterCmd)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after command, report timeout\n"));
					retState = ATA_STATE_TIMEOUT;
					}
				else if (SeqStatus.seqPos == seqErrAfterCmd)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after command, return device error\n"));
					retState = ATA_STATE_DEVICE_ERROR;
					}
				else if (SeqStatus.seqPos == seqDrqAfterCmd)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present after command, reset device\n"));
					retState = ATA_STATE_RESET_ERROR;
					}
				else 
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
					ASSERT(FALSE);
					retState = ATA_STATE_RESET_ERROR;
					}

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATA_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_IssueCmdSeq - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_IssueInDataSeq
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_IssueInDataSeq
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS 	ataRegs,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerBlock
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState = ATA_STATE_SUCCESS;
	NTSTATUS				wStatus;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerBlock);
	UCHAR 					hiXfer = (UCHAR)((bytesPerBlock-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerBlock-1);
   	UCHAR seqDataIn[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-02 */     (SEQ_RRMW|REG_CCTR), REG_CCTR_MANUAL_REVERSE, REG_CCTR_MANUAL_REVERSE,
/* 03-04 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 05-09 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 10-12 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 13-15 */		(SEQ_RRMW|REG_CTRL), REG_CTRL_DIRECTION_IN, REG_CTRL_DIRECTION_IN, // Set Direction bit to 1 (Input Mode)
/* 16-17 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 18-20 */		(SEQ_WBIB|0x00),     hiXfer, loXfer,
/* 21-23 */		(SEQ_RRMW|REG_CTRL), 0x00, REG_CTRL_DIRECTION_IN, 			// Set Direction bit to 0 (Output Mode)
/* 24-26 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 27-28 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 29-33 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 34-35 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 36-40 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_IssueInDataSeq\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;


	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqDataIn, sizeof(seqDataIn), dataBuffer, Mdl, dataTransferLength, TRUE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATA_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Data In Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATA_STATE_SUCCESS;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Data In Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);
				switch(SeqStatus.seqPos)
					{
					case 10:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATA_STATE_SUCCESS;
						break;

					case 34:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATA_STATE_TIMEOUT;
						break;

					case 41:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATA_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATA_STATE_RESET_ERROR;
						break;
					}
				
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per Block\n", blockCountProcessed, bytesPerBlock));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerBlock);

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATA_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_IssueInDataSeq - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_IssueOutDataSeq
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_IssueOutDataSeq
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS 	ataRegs,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength,
	USHORT				bytesPerBlock
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState = ATA_STATE_SUCCESS;
	NTSTATUS				wStatus;
	USHORT					numBlocks =	(USHORT)(dataTransferLength / bytesPerBlock);
	UCHAR 					hiXfer = (UCHAR)((bytesPerBlock-1) >> 8);
	UCHAR 					loXfer = (UCHAR)(bytesPerBlock-1);
   	UCHAR seqDataOut[] = 
   		{
        (UCHAR)(numBlocks-1), (UCHAR)((numBlocks-1)>>8), SEQ_BLOCKING_BIT,
/* 00-01 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 02-06 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, ATA_STATUS_DRQ, ATA_STATUS_DRQ,

/* 07-09 */		(SEQ_RRMW|REG_SETU), 0x00, REG_SETU_HOLD_BULK,
/* 10-11 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_DATA,
/* 12-14 */		(SEQ_WBOB|0x00),     hiXfer, loXfer,
/* 15-17 */		(SEQ_RRMW|REG_SETU), REG_SETU_HOLD_BULK, REG_SETU_HOLD_BULK,

/* 18-19 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_ALT_STATUS,
/* 20-24 */		(SEQ_RRCM|REG_EPPA), 0xFF, 0xFF, 0x00, ATA_STATUS_BUSY,
/* 25-26 */		(SEQ_RWRI|REG_ATAA), ATA_ADDRESS_STATUS,
/* 27-31 */		(SEQ_RRCM|REG_EPPA), 0x00, 0x00, 0x00, ATA_STATUS_ERROR,
        };

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_IssueOutDataSeq\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;


	wStatus = USBMASS_IssueSeqBulk(DeviceObject, seqDataOut, sizeof(seqDataOut), dataBuffer, Mdl, dataTransferLength, FALSE, 0);

	if (wStatus == STATUS_TIMEOUT)
		{
		retState = ATA_STATE_RESET_ERROR;
		}
	else
		{
		/* check status returned from both command and data IRP for success */
		if ((pFs->IoctlStatus == STATUS_SUCCESS) && (pFs->BulkStatus == STATUS_SUCCESS))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Data Out Request completed successfully\n"));
			pFs->BytesTransferred += dataTransferLength;
			retState = ATA_STATE_SUCCESS;
			}
		else
			{
			SEQUENCER_STATUS SeqStatus;
			USHORT blockCountProcessed;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Data Out Request failed\n"));

			if (USBGetSeqStatus(DeviceObject, &SeqStatus) == STATUS_SUCCESS)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqFlags = 0x%x\n", SeqStatus.statusFlags));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SeqPos   = 0x%x\n", SeqStatus.seqPos));

				/* check to see if sequencer is active */
				if (SeqStatus.statusFlags & SEQSTATUSF_ACTIVE)
					{
					/* abort sequencer */
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Aborting Sequencer\n"));
					USBAbortSeq(DeviceObject);
					}

		        blockCountProcessed = numBlocks - (((USHORT)SeqStatus.globalLoopCountLow | (USHORT)SeqStatus.globalLoopCountHigh << 8) + 1);
				switch(SeqStatus.seqPos)
					{
					case 07:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   DRQ not present before data xfer, no more data\n"));
						retState = ATA_STATE_SUCCESS;
						break;

					case 25:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device BUSY after data xfer, wait for not busy before complete\n"));
						blockCountProcessed += 1;
						retState = ATA_STATE_TIMEOUT;
						break;

					case 32:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR after data xfer, return device error\n"));
						blockCountProcessed += 1;
						retState = ATA_STATE_DEVICE_ERROR;
						break;

					default:
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Sequencer state, reset device\n"));
						ASSERT(FALSE);
						retState = ATA_STATE_RESET_ERROR;
						break;
					}
				
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processed %d blocks of %d bytes per Block\n", blockCountProcessed, bytesPerBlock));
				pFs->BytesTransferred += (blockCountProcessed * bytesPerBlock);

				USBMASS_ResetRegsForSeq(DeviceObject, TRUE);
				}
			else
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetSeqStatus call failed\n"));
				retState = ATA_STATE_RESET_ERROR;
				}
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_IssueOutDataSeq - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_PerformNDCmd
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_PerformNDCmd
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS ataRegs
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState;
    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_PerformNDCmd\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	do
		{
		retState = ATA_IssueCmdSeq(DeviceObject, ataRegs);

		if (retState == ATA_STATE_RESUBMIT)
			{
		    KIRQL irql;

			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					retState = ATA_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}
	while (retState == ATA_STATE_RESUBMIT);

	if (retState == ATA_STATE_TIMEOUT)
		{
		UCHAR state;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA State Timeout - Waiting for !BSY\n"));

		retState = ATA_STATE_SUCCESS;
		state = USBMASS_WaitForNotBusy(DeviceObject, 0);

		if (state == WAIT_STATE_RESET_ERROR)
			{	
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
			retState = ATA_STATE_RESET_ERROR;
			}
		else if (state == WAIT_STATE_DEVICE_ERROR)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
			retState = ATA_STATE_DEVICE_ERROR;
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_PerformNDCmd - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_PerformReadCmd
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_PerformReadCmd
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS 	ataRegs,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState;
    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_PerformReadCmd\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	do
		{
		retState = ATA_IssueCmdSeq(DeviceObject, ataRegs);

		if (retState == ATA_STATE_RESUBMIT)
			{
		    KIRQL irql;

			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					retState = ATA_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}
	while (retState == ATA_STATE_RESUBMIT);

	if (retState == ATA_STATE_TIMEOUT)
		{
		UCHAR state;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA State Timeout - Waiting for !BSY\n"));

		retState = ATA_STATE_SUCCESS;
		state = USBMASS_WaitForNotBusy(DeviceObject, 0);

		if (state == WAIT_STATE_RESET_ERROR)
			{	
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
			retState = ATA_STATE_RESET_ERROR;
			}
		else if (state == WAIT_STATE_DEVICE_ERROR)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
			retState = ATA_STATE_DEVICE_ERROR;
			}
		}

	if (retState == ATA_STATE_SUCCESS)
		{
		do
			{
			if (pFs->BytesTransferred)
				{
				if (dataBuffer)
					{
					PVOID buffer;

					buffer = (PUCHAR)dataBuffer + pFs->BytesTransferred;
					retState = ATA_IssueInDataSeq(DeviceObject, ataRegs, buffer, NULL, (dataTransferLength - pFs->BytesTransferred), ATA_BYTES_PER_BLOCK);
					}
				else
					{
				    PMDL  mdl;
				    PVOID mdlVa;

			        mdlVa = (PUCHAR)MmGetMdlVirtualAddress(Mdl) + pFs->BytesTransferred;

		            /* Part of a split request, allocate new partial MDL */
		            mdl = IoAllocateMdl(mdlVa,
		                                (dataTransferLength - pFs->BytesTransferred),
		                                FALSE,
		                                FALSE,
		                                NULL);
		            if (mdl == NULL)
			            {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
						retState = ATA_STATE_RESET_ERROR;
		    	        }
		            else
		        	    {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
		                IoBuildPartialMdl(Mdl,
		                                  mdl,
		                                  mdlVa,
		                                  (dataTransferLength - pFs->BytesTransferred));

						retState = ATA_IssueInDataSeq(DeviceObject, ataRegs, NULL, mdl, (dataTransferLength - pFs->BytesTransferred), ATA_BYTES_PER_BLOCK);

					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
						IoFreeMdl(mdl);
		            	}
					}
				}
			else
				{
				retState = ATA_IssueInDataSeq(DeviceObject, ataRegs, dataBuffer, Mdl, dataTransferLength, ATA_BYTES_PER_BLOCK);
				}

			if (retState == ATA_STATE_TIMEOUT)
				{
			    KIRQL irql;

				/* check for SRB timeout */
			    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
			        {
			        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
			    	    {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
						retState = ATA_STATE_RESET_ERROR;
						}
				    }
			    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
				}
			}
		while (retState == ATA_STATE_TIMEOUT);
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_PerformReadCmd - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_PerformWriteCmd
 *                                                                         
 * 
 *
 *
 * RETURNS:
 *    UCHAR
 */                                                                        
UCHAR
ATA_PerformWriteCmd
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PATACOMMANDREGS 	ataRegs,
	PVOID				dataBuffer,
	PMDL 				Mdl,
	ULONG	 			dataTransferLength
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
	UCHAR					retState;
    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_PerformWriteCmd\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	do
		{
		retState = ATA_IssueCmdSeq(DeviceObject, ataRegs);

		if (retState == ATA_STATE_RESUBMIT)
			{
		    KIRQL irql;

			/* check for SRB timeout */
		    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
		        {
		        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
		    	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
					retState = ATA_STATE_RESET_ERROR;
					}
			    }
		    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
			}
		}
	while (retState == ATA_STATE_RESUBMIT);

	if (retState == ATA_STATE_TIMEOUT)
		{
		UCHAR state;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA State Timeout - Waiting for !BSY\n"));

		retState = ATA_STATE_SUCCESS;
		state = USBMASS_WaitForNotBusy(DeviceObject, 0);

		if (state == WAIT_STATE_RESET_ERROR)
			{	
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   WaitForNotBusy failed, reset device\n"));
			retState = ATA_STATE_RESET_ERROR;
			}
		else if (state == WAIT_STATE_DEVICE_ERROR)
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device ERROR bit detected\n"));
			retState = ATA_STATE_DEVICE_ERROR;
			}
		}

	if (retState == ATA_STATE_SUCCESS)
		{
		do
			{
			if (pFs->BytesTransferred)
				{
				if (dataBuffer)
					{
					PVOID buffer;

					buffer = (PUCHAR)dataBuffer + pFs->BytesTransferred;
					retState = ATA_IssueInDataSeq(DeviceObject, ataRegs, buffer, NULL, (dataTransferLength - pFs->BytesTransferred), ATA_BYTES_PER_BLOCK);
					}
				else
					{
				    PMDL  mdl;
				    PVOID mdlVa;

			        mdlVa = (PUCHAR)MmGetMdlVirtualAddress(Mdl) + pFs->BytesTransferred;

		            /* Part of a split request, allocate new partial MDL */
		            mdl = IoAllocateMdl(mdlVa,
		                                (dataTransferLength - pFs->BytesTransferred),
		                                FALSE,
		                                FALSE,
		                                NULL);
		            if (mdl == NULL)
			            {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
						retState = ATA_STATE_RESET_ERROR;
		    	        }
		            else
		        	    {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
		                IoBuildPartialMdl(Mdl,
		                                  mdl,
		                                  mdlVa,
		                                  (dataTransferLength - pFs->BytesTransferred));

						retState = ATA_IssueOutDataSeq(DeviceObject, ataRegs, NULL, mdl, (dataTransferLength - pFs->BytesTransferred), ATA_BYTES_PER_BLOCK);

					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
						IoFreeMdl(mdl);
		            	}
					}
				}
			else
				{
				retState = ATA_IssueOutDataSeq(DeviceObject, ataRegs, dataBuffer, Mdl, dataTransferLength, ATA_BYTES_PER_BLOCK);
				}

			if (retState == ATA_STATE_TIMEOUT)
				{
			    KIRQL irql;

				/* check for SRB timeout */
			    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
			        {
			        if (!TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
			    	    {
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Timeout, reset device\n"));
						retState = ATA_STATE_RESET_ERROR;
						}
				    }
			    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);
				}
			}
		while (retState == ATA_STATE_TIMEOUT);
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_PerformWriteCmd - 0x%x\n", retState));
	return retState;
	}


/**************************************************************************
 * ATA_Request
 *                                                                         
 * Issue the ATA request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
ATA_Request
	(
    IN PDEVICE_OBJECT   DeviceObject,
	PVOID				dataBuffer,
	PMDL 				mdl,
	ULONG	 			dataTransferLength
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    BOOLEAN 				success = TRUE;
	UCHAR					curState = ATA_STATE_DEVICE_ERROR;
	ATACOMMANDREGS 			ataRegs;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering ATA_Request\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

    cdb = (PCDB)srb->Cdb;

    switch (srb->Cdb[0])
	    {
        case SCSIOP_INQUIRY:
			{
		    PINQUIRYDATA    pInquiry;
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_INQUIRY\n"));

			/* copy InquiryData */
	        srb->DataTransferLength = min(srb->DataTransferLength, sizeof(INQUIRYDATA));
	        memcpy(srb->DataBuffer, &pdoDeviceExtension->InquiryData, srb->DataTransferLength);

			pInquiry = (PINQUIRYDATA)srb->DataBuffer;

			if (fdoDeviceExtension->MountAsRemovable & MR_REMOVABLE)
				{	
				pInquiry->RemovableMedia = 1;
				}

			curState = ATA_STATE_SUCCESS;

	        pFs->BytesTransferred = srb->DataTransferLength;
			}
			break;

        case SCSIOP_MEDIUM_REMOVAL:
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_MEDIUM_REMOVAL\n"));

			if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_REMOVABLE_MEDIA))
				{
				ataRegs.regFlags = 0;
				ataRegs.cmdFlags = 0;
				ataRegs.cmd = (cdb->MEDIA_REMOVAL.Prevent == TRUE) ? ATA_COMMAND_DOOR_LOCK : ATA_COMMAND_DOOR_UNLOCK;
				curState = ATA_PerformNDCmd(DeviceObject, &ataRegs);
				}						
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Removeable Media not supported, just report okay\n"));
				curState = ATA_STATE_SUCCESS;
   				}
			break;

        case SCSIOP_MODE_SENSE:
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_MODE_SENSE\n"));

			if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_MEDIA_STATUS_ENABLED))
				{
				ataRegs.regFlags = 0;
				ataRegs.cmdFlags = 0;
				ataRegs.cmd = ATA_COMMAND_GET_MEDIA_STATUS;
				curState = ATA_PerformNDCmd(DeviceObject, &ataRegs);

				if (curState != ATA_STATE_SUCCESS)
					{
					UCHAR allRegs[8];

					/* read all registers */
					if (NT_SUCCESS(USBMASS_GetAllATARegs(DeviceObject, &allRegs[0])))
						{
						/* check for !BSY and ERR */
						if (allRegs[RS_STATUS] & ATA_STATUS_BUSY)
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Status register is BUSY???\n"));
							}
						else if (allRegs[RS_STATUS] & ATA_STATUS_ERROR)
							{
							/* check error register for write protected error */
							if (allRegs[RS_ERROR] & ATA_ERROR_DATA_ERROR)
								{
							    PMODE_PARAMETER_HEADER modeData;

								/* media is write-protected, set bit in mode sense buffer */
								modeData = (PMODE_PARAMETER_HEADER)srb->DataBuffer;

								srb->DataTransferLength = sizeof(MODE_PARAMETER_HEADER);
								modeData->DeviceSpecificParameter |= MODE_DSP_WRITE_PROTECT;

								curState = ATA_STATE_SUCCESS;
						        pFs->BytesTransferred = srb->DataTransferLength;
								}
							}
						}
					else
						{
					    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   GetAllATARegs failed, reset device\n"));
						curState = ATA_STATE_RESET_ERROR;
						}
					}
				}						
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Removeable Media not supported, just report okay\n"));
				curState = ATA_STATE_SUCCESS;
   				}
			break;

        case SCSIOP_TEST_UNIT_READY:
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_TEST_UNIT_READY\n"));

			if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_MEDIA_STATUS_ENABLED))
				{
				ataRegs.regFlags = 0;
				ataRegs.cmdFlags = 0;
				ataRegs.cmd = ATA_COMMAND_GET_MEDIA_STATUS;
				curState = ATA_PerformNDCmd(DeviceObject, &ataRegs);
				}						
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Removeable Media not supported, just report okay\n"));
				curState = ATA_STATE_SUCCESS;
				}
			break;

        case SCSIOP_READ_CAPACITY:
			{
			ULONG capacity;
	        READ_CAPACITY_DATA      readCapacityData;


		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_READ_CAPACITY\n"));

			/* detemine if it supports LBA */
			if (pdoDeviceExtension->IdentifyData.Capabilities & IDENTIFY_CAPABILITIES_LBA_SUPPORTED)
				{
				/* set last sector */
				capacity = pdoDeviceExtension->IdentifyData.UserAddressableSectors - 1;
				}
			else
				{
				/* calculate last sector */
				capacity = (pdoDeviceExtension->IdentifyData.NumberOfHeads *
							pdoDeviceExtension->IdentifyData.NumberOfCylinders *
							pdoDeviceExtension->IdentifyData.SectorsPerTrack) - 1;
				}

	        readCapacityData.LogicalBlockAddress = SWAP_DWORD(capacity);
	        readCapacityData.BytesPerBlock = 0x20000;
	        srb->DataTransferLength = min(srb->DataTransferLength, sizeof(READ_CAPACITY_DATA));
	        memcpy(srb->DataBuffer, &readCapacityData, srb->DataTransferLength);

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("  LogicalBlockAddress = 0x%x, BytesPerBlock = 0x%x\n", readCapacityData.LogicalBlockAddress, readCapacityData.BytesPerBlock));

			curState = ATA_STATE_SUCCESS;

	        pFs->BytesTransferred = srb->DataTransferLength;
			}
			break;

        case SCSIOP_READ:
			{
			UCHAR sectnum, head;
			USHORT cylinder;
	        ULONG lba;
	        ULONG blockCount;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_READ\n"));

			sectnum = 0x00;
			head = 0x00;
			cylinder = 0x0000;

	        lba = *(ULONG *)&srb->Cdb[2]; 
	        lba = SWAP_DWORD(lba);
	        blockCount = (ULONG)srb->Cdb[7]<<8 | (ULONG)srb->Cdb[8];

			/* detemine if it supports LBA */
			if ((pdoDeviceExtension->IdentifyData.Capabilities & IDENTIFY_CAPABILITIES_LBA_SUPPORTED) != 0)
				{
				sectnum = (UCHAR)(lba); // sectnum contains 0..7 of LBA
				cylinder = (USHORT)(lba>>8); // cylinder contains 8..23 of LBA
				head = ATA_ADDRESS_DEVHEAD_LBA_MODE | (UCHAR)(lba>>24 & 0x0F); // head contains 24..27 of LBA
				}
			else
				{
				sectnum = (UCHAR)((lba % pdoDeviceExtension->IdentifyData.SectorsPerTrack) + 1);
				cylinder = (USHORT)(lba / (pdoDeviceExtension->IdentifyData.SectorsPerTrack * pdoDeviceExtension->IdentifyData.NumberOfHeads));
				head = (UCHAR)((lba / pdoDeviceExtension->IdentifyData.SectorsPerTrack) % pdoDeviceExtension->IdentifyData.NumberOfHeads);
				}

			ataRegs.regFlags = (ATA_REGMASK_SECTOR_COUNT | ATA_REGMASK_SECTOR_NUM | ATA_REGMASK_CYLINDER | ATA_REGMASK_HEAD);
			ataRegs.cmdFlags = ATA_CMDMASK_WILL_XFER_DATA;
			ataRegs.sectorCount = (UCHAR)blockCount;
			ataRegs.sectorNum = sectnum;
			ataRegs.cylinder = cylinder;
			ataRegs.head = head;
			ataRegs.cmd = ATA_COMMAND_READ;

			curState = ATA_PerformReadCmd(DeviceObject, &ataRegs, dataBuffer, mdl, dataTransferLength);
			}
			break;


        case SCSIOP_WRITE:
        case SCSIOP_WRITE_VERIFY:
			{
			UCHAR sectnum, head;
			USHORT cylinder;
	        ULONG lba;
	        ULONG blockCount;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_WRITE/WRITE_VERIFY\n"));

			sectnum = 0x00;
			head = 0x00;
			cylinder = 0x0000;

	        lba = *(ULONG *)&srb->Cdb[2]; 
	        lba = SWAP_DWORD(lba);
	        blockCount = (ULONG)srb->Cdb[7]<<8 | (ULONG)srb->Cdb[8];

			/* detemine if it supports LBA */
			if ((pdoDeviceExtension->IdentifyData.Capabilities & IDENTIFY_CAPABILITIES_LBA_SUPPORTED) != 0)
				{
				sectnum = (UCHAR)(lba); // sectnum contains 0..7 of LBA
				cylinder = (USHORT)(lba>>8); // cylinder contains 8..23 of LBA
				head = ATA_ADDRESS_DEVHEAD_LBA_MODE | (UCHAR)(lba>>24 & 0x0F); // head contains 24..27 of LBA
				}
			else
				{
				sectnum = (UCHAR)((lba % pdoDeviceExtension->IdentifyData.SectorsPerTrack) + 1);
				cylinder = (USHORT)(lba / (pdoDeviceExtension->IdentifyData.SectorsPerTrack * pdoDeviceExtension->IdentifyData.NumberOfHeads));
				head = (UCHAR)((lba / pdoDeviceExtension->IdentifyData.SectorsPerTrack) % pdoDeviceExtension->IdentifyData.NumberOfHeads);
				}

			ataRegs.regFlags = (ATA_REGMASK_SECTOR_COUNT | ATA_REGMASK_SECTOR_NUM | ATA_REGMASK_CYLINDER | ATA_REGMASK_HEAD);
			ataRegs.cmdFlags = ATA_CMDMASK_WILL_XFER_DATA;
			ataRegs.sectorCount = (UCHAR)blockCount;
			ataRegs.sectorNum = sectnum;
			ataRegs.cylinder = cylinder;
			ataRegs.head = head;
			ataRegs.cmd = ATA_COMMAND_WRITE;

			curState = ATA_PerformWriteCmd(DeviceObject, &ataRegs, dataBuffer, mdl, dataTransferLength);
			}
			break;

        case SCSIOP_VERIFY:
			{
			UCHAR sectnum, head;
			USHORT cylinder;
	        ULONG lba;
	        ULONG blockCount;

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_VERIFY\n"));

			sectnum = 0x00;
			head = 0x00;
			cylinder = 0x0000;

	        lba = *(ULONG *)&srb->Cdb[2]; 
	        lba = SWAP_DWORD(lba);
	        blockCount = (ULONG)srb->Cdb[7]<<8 | (ULONG)srb->Cdb[8];

			/* detemine if it supports LBA */
			if ((pdoDeviceExtension->IdentifyData.Capabilities & IDENTIFY_CAPABILITIES_LBA_SUPPORTED) != 0)
				{
				sectnum = (UCHAR)(lba); // sectnum contains 0..7 of LBA
				cylinder = (USHORT)(lba>>8); // cylinder contains 8..23 of LBA
				head = ATA_ADDRESS_DEVHEAD_LBA_MODE | (UCHAR)(lba>>24 & 0x0F); // head contains 24..27 of LBA
				}
			else
				{
				sectnum = (UCHAR)((lba % pdoDeviceExtension->IdentifyData.SectorsPerTrack) + 1);
				cylinder = (USHORT)(lba / (pdoDeviceExtension->IdentifyData.SectorsPerTrack * pdoDeviceExtension->IdentifyData.NumberOfHeads));
				head = (UCHAR)((lba / pdoDeviceExtension->IdentifyData.SectorsPerTrack) % pdoDeviceExtension->IdentifyData.NumberOfHeads);
				}

			ataRegs.regFlags = (ATA_REGMASK_SECTOR_COUNT | ATA_REGMASK_SECTOR_NUM | ATA_REGMASK_CYLINDER | ATA_REGMASK_HEAD);
			ataRegs.cmdFlags = 0;
			ataRegs.sectorCount = (UCHAR)blockCount;
			ataRegs.sectorNum = sectnum;
			ataRegs.cylinder = cylinder;
			ataRegs.head = head;
			ataRegs.cmd = ATA_COMMAND_VERIFY;

			curState = ATA_PerformNDCmd(DeviceObject, &ataRegs);

			}
			break;

        case SCSIOP_START_STOP_UNIT:
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_START_STOP_UNIT\n"));

			if (((PCDB)srb->Cdb)->START_STOP.Start == 0 && (((PCDB)srb->Cdb)->START_STOP.LoadEject==1)) 
				{
				ataRegs.regFlags = 0;
				ataRegs.cmdFlags = 0;
				ataRegs.cmd = ATA_COMMAND_MEDIA_EJECT;
				curState = ATA_PerformNDCmd(DeviceObject, &ataRegs);
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Nothing to do, just report okay\n"));
				curState = ATA_STATE_SUCCESS;
				}
			}
			break;

        case SCSIOP_FORMAT_UNIT:
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - SCSIOP_FORMAT_UNIT\n"));

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Format Unit not supported, just report okay\n"));
			curState = ATA_STATE_SUCCESS;
			break;

        default:
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA OUT - Unsupported SCSIOP (0x%x)\n", srb->Cdb[0]));
			ASSERT(FALSE);
			curState = ATA_STATE_INVALID_REQUEST;
            break;
	    }

	if (curState != ATA_STATE_SUCCESS)
		{
		success = FALSE;

        Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR;
        srb->SrbStatus = SRB_STATUS_ERROR;
        pFs->BytesTransferred = 0; 

		if (curState == ATA_STATE_RESET_ERROR)
			{
	        srb->SrbStatus = SRB_STATUS_BUS_RESET;

			if (!NT_SUCCESS(USBMASS_PerformReset(DeviceObject)))
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Failed PerformReset routine\n"));
				ASSERT(FALSE);
				}
			}
		else if (curState == ATA_STATE_INVALID_REQUEST)
			{
	        srb->SrbStatus = SRB_STATUS_INVALID_REQUEST;
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving ATA_Request - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_IssueAtaRequest
 *                                                                         
 * Issue the ATA request
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_IssueAtaRequest
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    BOOLEAN 				success = FALSE;
	UCHAR					curState;
    PMDL                    mdl;
    PVOID                   mdlVa;
    PVOID                   transferBuffer;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_IssueAtaRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

    if ((Irp->MdlAddress != NULL) ||
        ((srb != pdoDeviceExtension->OriginalSrb) && (srb->DataBuffer != NULL)))
		{
		/* setup either transfer buffer or MDL for use */
	    mdl = NULL;
	    transferBuffer = NULL;

	    if (srb == pdoDeviceExtension->OriginalSrb)
		    {
	        /* Check to see if this request is part of a split request. */
	        mdlVa = MmGetMdlVirtualAddress(Irp->MdlAddress);

	        if (mdlVa == (PVOID)srb->DataBuffer)
		        {
	            /* Not part of a split request, use original MDL */
	            mdl = Irp->MdlAddress;
		        }
	        else
	    	    {
	            /* Part of a split request, allocate new partial MDL */
	            mdl = IoAllocateMdl(srb->DataBuffer,
	                                srb->DataTransferLength,
	                                FALSE,
	                                FALSE,
	                                NULL);
	            if (mdl == NULL)
		            {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate MDL\n"));
	    	        }
	            else
	        	    {
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Allocated new MDL, should be freed later\n"));
	                IoBuildPartialMdl(Irp->MdlAddress,
	                                  mdl,
	                                  srb->DataBuffer,
	                                  srb->DataTransferLength);
	            	}
		        }
	    	}
	    else
		    {
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Using SRB DataBuffer\n"));
	        transferBuffer = srb->DataBuffer;
	    	}

	    if (mdl != NULL || transferBuffer != NULL)
			{
			success = ATA_Request(DeviceObject, transferBuffer, mdl, srb->DataTransferLength);

			/* determine if we need to free an MDL we created */
		    if ((transferBuffer == NULL) && (srb == pdoDeviceExtension->OriginalSrb && mdl != Irp->MdlAddress))
			    {
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Free MDL\n"));
		        IoFreeMdl(mdl);
		    	}
			}
		else
			{
		    Irp->IoStatus.Status = STATUS_UNSUCCESSFUL;
			}
		}
	else
		{
		success = ATA_Request(DeviceObject, NULL, NULL, 0);
		}

	/* if request failed, then determine if Request Sense should be issued */
	if (!success)
		{
		/* check for SRB error */
		if (srb->SrbStatus == SRB_STATUS_ERROR)
			{
            Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR; /* ????? */
	        pFs->BytesTransferred = 0; /* ????? Leave as set? */
			}
		else
			{
			if (srb->SrbStatus == SRB_STATUS_BUS_RESET)
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device was Reset\n"));
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Unknown Device Error\n"));
				}

            Irp->IoStatus.Status = STATUS_IO_DEVICE_ERROR; /* ????? */
	        pFs->BytesTransferred = 0; /* ????? Leave as set? */
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_IssueAtaRequest - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_Scsi2Ata
 *                                                                         
 * Does any conversion from SCSI to ATA
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_Scsi2Ata
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    BOOLEAN 				success = TRUE;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Scsi2Ata\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

    cdb = (PCDB)srb->Cdb;

	/* setup internal CDB buffer */
	RtlZeroMemory(pFs->CdbBuffer, sizeof(pFs->CdbBuffer));
	RtlCopyMemory(pFs->CdbBuffer, srb->Cdb, srb->CdbLength);

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Scsi2Ata - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_Ata2Scsi
 *                                                                         
 * Does any conversion from ATA to SCSI
 *
 *
 * RETURNS:
 *    BOOLEAN
 */                                                                        
BOOLEAN
USBMASS_Ata2Scsi
	(
    IN PDEVICE_OBJECT   DeviceObject
    )
	{
    IN PIRP             	Irp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
	PCUSTOMPARAM 				pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    BOOLEAN 				success = TRUE;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Ata2Scsi\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

	Irp = pFs->SrbIrp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;

    cdb = (PCDB)srb->Cdb;

    switch (srb->Cdb[0])
	    {
        default:
            break;
	    }

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Ata2Scsi - %d\n", success));
	return success;
	}


/**************************************************************************
 * USBMASS_ProcessSRB
 *                                                                         
 * Starts processing the SRB
 *
 * This routine is called by USBMASS_StartIo().
 *
 * It runs at DPC level.
 *
 * RETURNS:
 *    N/A
 */                                                                        
VOID
USBMASS_ProcessSRB
	(
    IN PQUEUE_WORKITEM workItem
    )
	{
	PDEVICE_OBJECT   		DeviceObject = workItem->DeviceObject;
    IN PIRP             	Irp = workItem->SrbIrp;
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
	PCUSTOMPARAM 			pFs;
    PIO_STACK_LOCATION      irpStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    KIRQL 					irql;

    
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_ProcessSRB\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

	pFs->SrbIrp = Irp;

    /* Get the client Srb */
    irpStack = IoGetCurrentIrpStackLocation(Irp);
    srb = irpStack->Parameters.Scsi.Srb;
    pdoDeviceExtension->OriginalSrb = srb;

    cdb = (PCDB)srb->Cdb;

	pFs->BytesTransferred = 0;
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;
	srb->SrbStatus = SRB_STATUS_SUCCESS;
	srb->ScsiStatus = SCSISTAT_GOOD;

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SCSI CDB OP - %s\n", ScsiCdbString(srb->Cdb[0])));
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   CDB (0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x-0x%x)\n",
										srb->Cdb[0],
										srb->Cdb[1],
										srb->Cdb[2],
										srb->Cdb[3],
										srb->Cdb[4],
										srb->Cdb[5],
										srb->Cdb[6],
										srb->Cdb[7],
										srb->Cdb[8],
										srb->Cdb[9],
										srb->Cdb[10],
										srb->Cdb[11]));
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Target ID = %d\n", srb->TargetId));

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SCSI CDB Length = 0x%x\n", srb->CdbLength));
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SCSI Timeout = %d seconds\n", srb->TimeOutValue));

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Transfer Info:\n"));
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      Data Transfer Length = 0x%x\n", srb->DataTransferLength));
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      MDL Address = 0x%x\n", Irp->MdlAddress));
    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      Data Buffer = 0x%x\n", srb->DataBuffer));

    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
        {
		SET_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS);
		
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Set SRB In Progress Flag\n"));
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

	    }
    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);

	/* check for an ATA device */
	if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_ATA_DEVICE))
		{
        USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processing ATA Request\n"));

		/* do any neccessary SCSI to ATA conversions */
		if (USBMASS_Scsi2Ata(DeviceObject))
			{
			/* issue the ATA request */
			if (USBMASS_IssueAtaRequest(DeviceObject))
				{
				/* do any ATA to SCSI conversion */
				USBMASS_Ata2Scsi(DeviceObject);
				}
			}
		}
	else
		{
        USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Processing ATAPI Request\n"));

		/* do any neccessary SCSI to ATAPI conversions */
		if (USBMASS_Scsi2Atapi(DeviceObject))
			{
			/* issue the ATAPI request */
			if (USBMASS_IssueAtapiRequest(DeviceObject))
				{
				/* do any ATAPI to SCSI conversion */
				USBMASS_Atapi2Scsi(DeviceObject);
				}
			}
		}

    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
        {
        if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS))
			{
		    CLEAR_FLAG(pdoDeviceExtension->DeviceFlags, DF_SRB_IN_PROGRESS);
		    
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Cleared SRB In Progress Flag\n"));
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

			}
	    }
    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);

	if (Irp->IoStatus.Status == STATUS_SUCCESS)
		{
	    Irp->IoStatus.Information = srb->DataTransferLength = pFs->BytesTransferred;
		}
	else
		{
	    Irp->IoStatus.Information = srb->DataTransferLength = 0;
		}

	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   IRP Status  = 0x%x (Length = 0x%x)\n", Irp->IoStatus.Status, Irp->IoStatus.Information));
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SRB Status  = 0x%x (Length = 0x%x)\n", srb->SrbStatus, srb->DataTransferLength));
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   SCSI Status = 0x%x\n", srb->ScsiStatus));

	/* Start the next request */
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Start Next Packet\n"));
	IoStartNextPacket(DeviceObject, FALSE);

	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Complete IRP\n"));
	IoCompleteRequest(Irp, IO_NO_INCREMENT);

	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Freeing Work Item\n"));
    ExFreePool(workItem);

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_ProcessSRB\n"));
	}


/*******************/
/* Common Routines */
/*******************/

/**************************************************************************
 * USBMASS_ManualEnum
 *                                                                         
 * Manually enumerate the ATA/ATAPI device
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_ManualEnum
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus = STATUS_SUCCESS;
    PVOID                   dataBuffer;
    ULONG                   dataTransferLength;
	ATACDB 					ataCdb;
	LARGE_INTEGER			startTimeout;
	UCHAR 					regValue;
	UCHAR 					allRegs[8];
	UCHAR					master_slave;
	PURB 					urb;
	BOOLEAN 				recheckAsMaster = FALSE;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_ManualEnum\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* default to master */
	master_slave = ATA_ADDRESS_DEVHEAD_STD;
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to master\n"));

	/* obtain current tick count, used to determine the timeout for BSY check */
    KeQueryTickCount(&startTimeout);

	/* loop until we detect !BSY or timeout */
	while(TRUE)
		{
		/* set Device/Head register */
		ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_HEAD, master_slave);

		/* check to see if this request failed */
	    if (!NT_SUCCESS(ntStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error setting Device/Head register\n"));
			// need to reset device here, just break for now
			break;
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to 0x%x\n", master_slave));
			}


		/* read Status register */
		ntStatus = USBMASS_GetATAReg(DeviceObject, ATA_ADDRESS_STATUS, &regValue);

		/* check to see if this request failed */
	    if (!NT_SUCCESS(ntStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error getting Status Register\n"));
			break;
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Got Status Register of 0x%x\n", regValue));
			}

		/* check for !BSY */
		if (regValue & ATA_STATUS_BUSY)
			{
			if (regValue == 0xFF)
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status is 0xFF and master device is probably not attached, continue with next operation\n"));
				break;
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status is still BSY, try again...\n"));
				}
			}
		else
			{
			/* status is !BSY */
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status !BSY, continue with next operation\n"));
			break;
			}

		/* check for timeout on this request */
		if (USBMASS_ElapsedTimeMs(&startTimeout) >= USBMASS_ENUM_BSY_TIMEOUT) 
			{
		    USBMASS_KdPrint(DBGLVL_MEDIUM, ("   BSY check timeout, just continue with next operation...\n"));
			}
		}

	if (NT_SUCCESS(ntStatus))
		{
		/* set to slave */
		master_slave ^= ATA_ADDRESS_DEVHEAD_SLAVE;
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to slave\n"));

		/* loop until we detect !BSY or timeout */
		while(TRUE)
			{
			/* set Device/Head register */
			ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_HEAD, master_slave);

			/* check to see if this request failed */
		    if (!NT_SUCCESS(ntStatus))
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error setting Device/Head register\n"));
				// need to reset device here, just break for now
				break;
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to 0x%x\n", master_slave));
				}


			/* read Status register */
			ntStatus = USBMASS_GetATAReg(DeviceObject, ATA_ADDRESS_STATUS, &regValue);

			/* check to see if this request failed */
		    if (!NT_SUCCESS(ntStatus))
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error getting Status Register\n"));
				break;
				}
			else
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Got Status Register of 0x%x\n", regValue));
				}

			/* check for !BSY */
			if (regValue & ATA_STATUS_BUSY)
				{
				if (regValue == 0xFF)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status is 0xFF and slave device is probably not attached, continue with next operation\n"));
					break;
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status is still BSY, try again...\n"));
					}
				}
			else
				{
				/* status is !BSY */
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status !BSY, continue with next operation\n"));
				break;
				}

			/* check for timeout on this request */
			if (USBMASS_ElapsedTimeMs(&startTimeout) >= USBMASS_ENUM_BSY_TIMEOUT) 
				{
			    USBMASS_KdPrint(DBGLVL_MEDIUM, ("   BSY check timeout, just continue with next operation...\n"));
				break;
				}
			}
		}

	/* set default back to master */
	master_slave = ATA_ADDRESS_DEVHEAD_STD;
	USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to master\n"));

	if (NT_SUCCESS(ntStatus))
		{
		/* perform SRST */
		ntStatus = USBMASS_IssueSRST(DeviceObject);

		if (NT_SUCCESS(ntStatus))
			{
			/* obtain current tick count, used to determine the timeout for detecting device */
		    KeQueryTickCount(&startTimeout);

			/* loop until we detect a device, timeout or error */
			while(TRUE)
				{
				/* set Device/Head register */
				ntStatus = USBMASS_SetATAReg(DeviceObject, ATA_ADDRESS_DEV_HEAD, master_slave);

				/* check to see if this request failed */
			    if (!NT_SUCCESS(ntStatus))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error setting Device/Head register\n"));
					// need to reset device here, just break for now
					break;
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Set Device/Head register to 0x%x\n", master_slave));
					}


				/* read Status register */
				ntStatus = USBMASS_GetAllATARegs(DeviceObject, &allRegs[0]);

			    if (!NT_SUCCESS(ntStatus))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Error getting Status Register\n"));
					break;
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Got Status Register of 0x%x\n", allRegs[REG_ALT_STATUS]));
					}

				/* check for BSY and */
				/* ATA_STATUS_DEVICE_FAULT (workaround ATA Zip drive) and */ 
				/* ATA_STATUS_ERROR (workaround for Archos CD-ROM) */
				if (allRegs[REG_ALT_STATUS] & (ATA_STATUS_BUSY | ATA_STATUS_DEVICE_FAULT | ATA_STATUS_ERROR))
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Status indicates it is not ready, try again...\n"));
					}
				else
					{
					/* check for DRDY, ATA devices set DRDY after SRST */
					if (allRegs[REG_ALT_STATUS] & ATA_STATUS_DRDY)
						{
						/* It seems that some devices will report itself okay as a slave when it is   */
						/* really a master.  So this check again as a master device just to make sure */
						/* it doesn't report itself okay as a master also.                            */
		                if ((master_slave & ATA_ADDRESS_DEVHEAD_SLAVE) && (recheckAsMaster == FALSE)) 
		                	{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identified ATA device as slave.  Rechecking again as master\n"));
		                    recheckAsMaster = TRUE;
			                }
						else
							{
						    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identified ATA device in %dms\n", USBMASS_ElapsedTimeMs(&startTimeout)));
							SET_FLAG(pdoDeviceExtension->DeviceFlags, DF_ATA_DEVICE);
							
							USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Set ATA Flag\n"));
							USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

							pdoDeviceExtension->DeviceHead = master_slave;

				            pdoDeviceExtension->SrbTimeout = USBMASS_DEFAULT_TIMEOUT;
							ntStatus = USBMASS_Identify(DeviceObject, &pdoDeviceExtension->IdentifyData, NULL, IDENTIFY_DATA_SIZE, FALSE);
							break;
							}
						}
					else
						{
						/* check Cylinder High/Low to determine it it is an ATAPI device */
						if ((allRegs[REG_CYLINDER_HIGH] == 0xEB) && (allRegs[REG_CYLINDER_LOW] == 0x14))
							{
							/* It seems that some devices will report itself okay as a slave when it is   */
							/* really a master.  So this check again as a master device just to make sure */
							/* it doesn't report itself okay as a master also.                            */
			                if ((master_slave & ATA_ADDRESS_DEVHEAD_SLAVE) && (recheckAsMaster == FALSE)) 
			                	{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identified ATAPI device as slave.  Rechecking again as master\n"));
			                    recheckAsMaster = TRUE;
				                }
							else
								{
							    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identified ATAPI device in %dms\n", USBMASS_ElapsedTimeMs(&startTimeout)));
								pdoDeviceExtension->DeviceHead = master_slave;

								ntStatus = USBMASS_AtapiSoftReset(DeviceObject);
							    if (NT_SUCCESS(ntStatus))
									{
						            pdoDeviceExtension->SrbTimeout = USBMASS_DEFAULT_TIMEOUT;
									ntStatus = USBMASS_Identify(DeviceObject, &pdoDeviceExtension->IdentifyData, NULL, IDENTIFY_PACKET_DATA_SIZE, TRUE);
								    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATAPI Identify Data:\n"));
									}
								break;
								}
							}
						}
					}

				/* check for timeout on this request */
				if (USBMASS_ElapsedTimeMs(&startTimeout) >= USBMASS_ENUM_DETECT_TIMEOUT) 
					{
				    USBMASS_KdPrint(DBGLVL_HIGH, ("   Device detect timeout!\n"));
					ntStatus = STATUS_TIMEOUT;
					break;
					}

				master_slave ^= ATA_ADDRESS_DEVHEAD_SLAVE;
				}		
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_ManualEnum %08X\n", ntStatus));
    return ntStatus;
	}


/**************************************************************************
 * USBMASS_GetInquiryData
 *                                                                         
 * Add description here
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_GetInquiryData
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
    NTSTATUS                ntStatus = STATUS_SUCCESS;
	int						i;
    PVOID                   dataBuffer;
    ULONG                   dataTransferLength;
    CDB                     cdb;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("\nEntering USBMASS_GetInquiryData\n"));

#if DBG	
	gDebugLevel = DBGLVL_HIGH;
#endif

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);
    
	/* Initalize 725 device */
	USBMASS_725Init(DeviceObject);

	/* set default to Master */
	pdoDeviceExtension->DeviceHead = ATA_ADDRESS_DEVHEAD_STD;

	/* attempt to manually enumerate this device */
	ntStatus = USBMASS_ManualEnum(DeviceObject);

    if (NT_SUCCESS(ntStatus))
		{
		/* check for an ATA device */
		if (TEST_FLAG(pdoDeviceExtension->DeviceFlags, DF_ATA_DEVICE))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Located an ATA device\n"));

#if DBG
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   ATA Command Identify Dump:\n"));

		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Identify Data Structure:\n"));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      GeneralConfiguration = 0x%x\n", pdoDeviceExtension->IdentifyData.GeneralConfiguration));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      NumberOfCylinders = 0x%x\n", pdoDeviceExtension->IdentifyData.NumberOfCylinders));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      NumberOfHeads = 0x%x\n", pdoDeviceExtension->IdentifyData.NumberOfHeads));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      UnformattedBytesPerTrack = 0x%x\n", pdoDeviceExtension->IdentifyData.UnformattedBytesPerTrack));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      UnformattedBytesPerSector = 0x%x\n", pdoDeviceExtension->IdentifyData.UnformattedBytesPerSector));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      SectorsPerTrack = 0x%x\n", pdoDeviceExtension->IdentifyData.SectorsPerTrack));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      SerialNumber[0] = 0x%x\n", pdoDeviceExtension->IdentifyData.SerialNumber[0]));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      BufferType = 0x%x\n", pdoDeviceExtension->IdentifyData.BufferType));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      BufferSectorSize = 0x%x\n", pdoDeviceExtension->IdentifyData.BufferSectorSize));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      NumberOfEccBytes = 0x%x\n", pdoDeviceExtension->IdentifyData.NumberOfEccBytes));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      FirmwareRevision[0] = 0x%x\n", pdoDeviceExtension->IdentifyData.FirmwareRevision[0]));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      ModelNumber[0] = 0x%x\n", pdoDeviceExtension->IdentifyData.ModelNumber[0]));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      MaximumBlockTransfer = 0x%x\n", pdoDeviceExtension->IdentifyData.MaximumBlockTransfer));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      DoubleWordIo = 0x%x\n", pdoDeviceExtension->IdentifyData.DoubleWordIo));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      Capabilities = 0x%x\n", pdoDeviceExtension->IdentifyData.Capabilities));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      PioCycleTimingMode = 0x%x\n", pdoDeviceExtension->IdentifyData.PioCycleTimingMode));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      DmaCycleTimingMode = 0x%x\n", pdoDeviceExtension->IdentifyData.DmaCycleTimingMode));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      TranslationFieldsValid = 0x%x\n", pdoDeviceExtension->IdentifyData.TranslationFieldsValid));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      NumberOfCurrentCylinders = 0x%x\n", pdoDeviceExtension->IdentifyData.NumberOfCurrentCylinders));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      NumberOfCurrentHeads = 0x%x\n", pdoDeviceExtension->IdentifyData.NumberOfCurrentHeads));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      CurrentSectorsPerTrack = 0x%x\n", pdoDeviceExtension->IdentifyData.CurrentSectorsPerTrack));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      CurrentSectorCapacity = 0x%x\n", pdoDeviceExtension->IdentifyData.CurrentSectorCapacity));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      CurrentMultiSectorSetting = 0x%x\n", pdoDeviceExtension->IdentifyData.CurrentMultiSectorSetting));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      UserAddressableSectors = 0x%x\n", pdoDeviceExtension->IdentifyData.UserAddressableSectors));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      CommandSetsSupported = 0x%x\n", pdoDeviceExtension->IdentifyData.CommandSetsSupported));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      CurrentAPMValue = 0x%x\n", pdoDeviceExtension->IdentifyData.CurrentAPMValue));
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("      MediaStatusNotification = 0x%x\n", pdoDeviceExtension->IdentifyData.MediaStatusNotification));
#endif

            /* Zero Inquiry Data */
			RtlZeroMemory(&pdoDeviceExtension->InquiryData, 
							sizeof(pdoDeviceExtension->InquiryData));

            /* Standard IDE interface only supports disks */
            pdoDeviceExtension->InquiryData.DeviceType = DIRECT_ACCESS_DEVICE;

			if (pdoDeviceExtension->IdentifyData.MediaStatusNotification & 1)
				{
	            /* set the removable bit */
				pdoDeviceExtension->InquiryData.RemovableMedia = 1;
				SET_FLAG(pdoDeviceExtension->DeviceFlags, DF_REMOVABLE_MEDIA);
				
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Set Removable Flag\n"));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

				}

			pdoDeviceExtension->InquiryData.AdditionalLength = 0x1F;

            /* Fill in vendor identification fields */
            for (i = 0; i < 20; i += 2) 
            	{
				pdoDeviceExtension->InquiryData.VendorId[i] =
                	((PUCHAR)pdoDeviceExtension->IdentifyData.ModelNumber)[i + 1];
				pdoDeviceExtension->InquiryData.VendorId[i+1] =
                	((PUCHAR)pdoDeviceExtension->IdentifyData.ModelNumber)[i];
	            }

            /* Initialize unused portion of product id */
            for (i = 0; i < 4; i++) 
				{
				pdoDeviceExtension->InquiryData.ProductId[12+i] = ' ';
				}

            /* Move firmware revision from IDENTIFY data to */
            /* product revision in INQUIRY data             */
            for (i = 0; i < 4; i += 2) 
            	{
               pdoDeviceExtension->InquiryData.ProductRevisionLevel[i] =
                   ((PUCHAR)pdoDeviceExtension->IdentifyData.FirmwareRevision)[i+1];
               pdoDeviceExtension->InquiryData.ProductRevisionLevel[i+1] =
                   ((PUCHAR)pdoDeviceExtension->IdentifyData.FirmwareRevision)[i];
				}

			/* determine if it supports Media Status Notification */
	        if (pdoDeviceExtension->IdentifyData.MediaStatusNotification & 1) 
	        	{
				ATACOMMANDREGS 			ataRegs;
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device supports Media Status Notification\n"));

				/* default to not support Media Status */
				CLEAR_FLAG(pdoDeviceExtension->DeviceFlags, DF_MEDIA_STATUS_ENABLED);
				
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Cleared Media Status Enabled Flag\n"));
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

				ataRegs.regFlags = ATA_REGMASK_FEATURES;
				ataRegs.features = ATA_FEATURE_ENABLE_MEDIA_STATUS;
				ataRegs.cmd = ATA_COMMAND_SET_FEATURES;
				if (ATA_PerformNDCmd(DeviceObject, &ataRegs) == ATA_STATE_SUCCESS)
					{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Enabled Media Status Notification\n"));
					SET_FLAG(pdoDeviceExtension->DeviceFlags, DF_MEDIA_STATUS_ENABLED);
					
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Set Media Status Enabled Flag\n"));
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("	 Device Flags are: 0x%x, Addr 0x%x\n", pdoDeviceExtension->DeviceFlags, &pdoDeviceExtension->DeviceFlags));

					}
				}
									
			/* determine if APM supported and enable FSL to spin down */	
			if (fdoDeviceExtension->DriverFlags & DF_CUST_APM_SUPPORT)
				{
				USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Driver flag for APM support is set, checking for device support\n")); 
				if (pdoDeviceExtension->IdentifyData.CommandSetsSupported & 0x8)
					{
					ATACOMMANDREGS 			ataRegs;
					USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Device supports APM, enabling with value %x\n", fdoDeviceExtension->APMState)); 
									
					ataRegs.regFlags = (ATA_REGMASK_FEATURES|ATA_REGMASK_SECTOR_COUNT);
					ataRegs.features = ATA_FEATURE_ENABLE_ADV_POWER;
					ataRegs.sectorCount = fdoDeviceExtension->APMState;
					ataRegs.cmd = ATA_COMMAND_SET_FEATURES;
					}
				}			
			
			}
		else
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Located an ATAPI device\n"));

			/* perform ATAPI Inquiry */
		    dataBuffer = (PVOID)&pdoDeviceExtension->InquiryData;
	    	dataTransferLength = sizeof(pdoDeviceExtension->InquiryData);

		    RtlZeroMemory(&cdb, sizeof(CDB));

		    cdb.CDB6INQUIRY.OperationCode = SCSIOP_INQUIRY;
		    cdb.CDB6INQUIRY.AllocationLength = (UCHAR)dataTransferLength;

		    ntStatus = USBMASS_IssueInternalCdb(DeviceObject,
		                                        dataBuffer,
		                                        &dataTransferLength,
		                                        &cdb,
		                                        sizeof(cdb.CDB6INQUIRY),
												TRUE,
		                                        20,
		                                        3);
			}
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_GetInquiryData 0x%08X\n\n", ntStatus));

    return ntStatus;
	}


/****************/
/* USB Routines */
/****************/

/**************************************************************************
 * USBBuildVendorOrClassRequest
 *                                                                         
 * Builds a vendor or class specific URB (USB Request Block).
 *
 * RETURNS:
 *    Address of allocated URB if success, or NULL if failure.
 */                                                                        
PURB USBBuildVendorOrClassRequest
	(
		IN USHORT Function,
		IN ULONG  TransferFlags,
		IN ULONG  TransferBufferLength,
		IN PVOID  TransferBuffer,
		IN UCHAR  RequestTypeReservedBits,
		IN UCHAR  Request,
		IN USHORT Value,
		IN USHORT Index
	) 
	{
    USHORT siz;
    ULONG length;
    PURB urb = NULL;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBBuildVendorOrClassRequest\n"));

    /* Size of the urb */
	siz = sizeof ( struct _URB_CONTROL_VENDOR_OR_CLASS_REQUEST );

    /* Allocate nonpaged memory to urb               */
	/* Note: callers must free the urb by themselves */
	urb = ExAllocatePoolWithTag(NonPagedPool,
                                siz,
                                POOL_TAG);

    if ( urb )
		{
        /* Initialize the urb */
		RtlZeroMemory(urb, siz);

		urb->UrbControlVendorClassRequest.Hdr.Length = siz;
		urb->UrbControlVendorClassRequest.Hdr.Function = Function;
		urb->UrbControlVendorClassRequest.TransferFlags = TransferFlags;
		urb->UrbControlVendorClassRequest.TransferBufferLength = TransferBufferLength;
		urb->UrbControlVendorClassRequest.TransferBuffer = TransferBuffer;
		urb->UrbControlVendorClassRequest.TransferBufferMDL = NULL;
		urb->UrbControlVendorClassRequest.UrbLink = NULL;
		urb->UrbControlVendorClassRequest.RequestTypeReservedBits = RequestTypeReservedBits;
		urb->UrbControlVendorClassRequest.Request = Request;
		urb->UrbControlVendorClassRequest.Value	= Value;
		urb->UrbControlVendorClassRequest.Index	= Index;
		}

	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBBuildVendorOrClassRequest\n"));
    return urb;
	}

/**************************************************************************
 * USBBuildVendorOrClassRequest
 *                                                                         
 * Builds a USB Bulk URB (USB Request Block).
 *
 * RETURNS:
 *    Address of allocated URB if success, or NULL if failure.
 */                                                                        
PURB USBBuildBulkRequest
	(
		IN ULONG  	TransferBufferLength,
		IN PVOID  	TransferBuffer,
		IN PMDL   	TransferBufferMDL,
		IN USBD_PIPE_HANDLE	PipeHandle,
		IN BOOLEAN 	Read
	) 
	{
    USHORT siz;
    ULONG length;
    PURB urb = NULL;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBBuildBulkRequest\n"));

    /* Size of the urb */
	siz = sizeof ( struct _URB_BULK_OR_INTERRUPT_TRANSFER );

    /* Allocate nonpaged memory to urb               */
	/* Note: callers must free the urb by themselves */
	urb = ExAllocatePoolWithTag(NonPagedPool,
                                siz,
                                POOL_TAG);

    if ( urb )
	{
        /* Initialize the urb */
		RtlZeroMemory(urb, siz);

        urb->UrbBulkOrInterruptTransfer.Hdr.Length = (USHORT) siz;

		urb->UrbBulkOrInterruptTransfer.Hdr.Function = 
			URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER;

		urb->UrbBulkOrInterruptTransfer.TransferFlags =
            Read ? (USBD_TRANSFER_DIRECTION_IN | USBD_SHORT_TRANSFER_OK) : 0;

	    urb->UrbBulkOrInterruptTransfer.PipeHandle = PipeHandle;

        urb->UrbBulkOrInterruptTransfer.UrbLink = NULL;

		urb->UrbBulkOrInterruptTransfer.TransferBufferLength = TransferBufferLength;

		urb->UrbBulkOrInterruptTransfer.TransferBuffer = TransferBuffer;

		urb->UrbBulkOrInterruptTransfer.TransferBufferMDL = TransferBufferMDL;

	}

	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBBuildBulkRequest\n"));
    return urb;
	}

/**************************************************************************
 * USBIoctlCompletionRoutine
 *                                                                         
 * Completion routine for USB Ioctl requests.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBIoctlCompletionRoutine 
	(
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp,
    PCOMPLETION_CONTEXT Context
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PCUSTOMPARAM pFs;
    KIRQL irql;


    USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBIoctlCompletionRoutine\n"));

    pdoDeviceExtension = Context->DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

    USBMASS_KdPrint(DBGLVL_HIGH, ("   Set Event\n"));
    KeSetEvent(Context->Event,
               IO_NO_INCREMENT,
               FALSE);

    USBMASS_KdPrint(DBGLVL_HIGH, ("   Ioctl Completion Status = 0x%x\n", Irp->IoStatus.Status));
	pFs->IoctlStatus = Irp->IoStatus.Status;

    if (Context->Irp)
    	{
	    USBMASS_KdPrint(DBGLVL_HIGH, ("   Freeing Irp\n"));
    	IoFreeIrp(Context->Irp);
		Context->Irp = NULL;
		}

	if (Context->Urb)
    	{
	    USBMASS_KdPrint(DBGLVL_HIGH, ("   Freeing Urb\n"));
        ExFreePool(Context->Urb);
		Context->Urb = NULL;
	    }

    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
	    {
		pFs->IoctlIrp = NULL;

	    if (!NT_SUCCESS(pFs->IoctlStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bad Ioctl Status = 0x%x\n", pFs->IoctlStatus));
			if (pFs->BulkIrp && !pFs->Read)
				{
				    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Bulk IRP\n"));
			    	IoCancelIrp(pFs->BulkIrp);
				}
			}
	    }
    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);

    USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBIoctlCompletionRoutine %08X\n", STATUS_MORE_PROCESSING_REQUIRED));

    return STATUS_MORE_PROCESSING_REQUIRED;
}

/**************************************************************************
 * USBIoctlRequest
 *                                                                         
 * Issues a USB Ioctl request.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBIoctlRequest 
	(
    PDEVICE_OBJECT  DeviceObject,
    PURB            Urb,
	LONG			IrpTimeout
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION   fdoDeviceExtension;
	PCUSTOMPARAM pFs;
    PIRP                    irp;
    PIO_STACK_LOCATION      nextStack;
    NTSTATUS                ntStatus;
    PCOMPLETION_CONTEXT context;


    PAGED_CODE();

    USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBIoctlRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

    context = &pFs->ioctlContext;

	if (context)
		{
	    /* Initialize the event we'll wait on */
	    KeInitializeEvent(&pFs->IoctlEvent,
	                      SynchronizationEvent,
	                      FALSE);

	    /* Allocate the Irp */
	    irp = IoAllocateIrp(fdoDeviceExtension->StackDeviceObject->StackSize, FALSE);

	    if (irp == NULL)
		    {
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBIoctlRequest %08X\n", STATUS_INSUFFICIENT_RESOURCES));
	        return STATUS_INSUFFICIENT_RESOURCES;
	    	}

	    /* Set the Irp parameters */
	    nextStack = IoGetNextIrpStackLocation(irp);
	    nextStack->MajorFunction = IRP_MJ_INTERNAL_DEVICE_CONTROL;
	    nextStack->Parameters.DeviceIoControl.IoControlCode =
	        IOCTL_INTERNAL_USB_SUBMIT_URB;
	    nextStack->Parameters.Others.Argument1 = Urb;

		context->DeviceObject = DeviceObject;
		context->Event = &pFs->IoctlEvent;
		context->Urb = Urb;
		context->Irp = irp;

		pFs->IoctlStatus = STATUS_SUCCESS;
		pFs->IoctlIrp = irp;

	    /* Set the completion routine, which will signal the event */
	    IoSetCompletionRoutine(irp,
	                           USBIoctlCompletionRoutine,
	                           context,
	                           TRUE,    // InvokeOnSuccess
	                           TRUE,    // InvokeOnError
	                           TRUE);   // InvokeOnCancel



	    /* Pass the Irp & Urb down the stack */
	    ntStatus = IoCallDriver(fdoDeviceExtension->StackDeviceObject,
	                            irp);

	    /* If the request is pending, block until it completes */
	    if (ntStatus == STATUS_PENDING)
		    {
			if (IrpTimeout)
				{
				if (IrpTimeout != IRP_INFINITE)
					{
			        LARGE_INTEGER timeout;

			        /* Specify a timeout of 5 seconds to wait for this call to complete. */
			        timeout.QuadPart = -10000 * IrpTimeout;

				    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
			        ntStatus = KeWaitForSingleObject(&pFs->IoctlEvent,
			                                         Executive,
			                                         KernelMode,
			                                         FALSE,
			                                         &timeout);

			        if (ntStatus == STATUS_TIMEOUT)
				        {
					    USBMASS_KdPrint(DBGLVL_HIGH, ("   Timed out... Cancelling request\n"));

			            ntStatus = STATUS_IO_TIMEOUT;

						if (irp)
							{
				            /* Cancel the Irp we just sent. */
				            IoCancelIrp(irp);
							}

					    USBMASS_KdPrint(DBGLVL_HIGH, ("   Waiting for request to complete\n"));

			            /* And wait until the cancel completes */
			            KeWaitForSingleObject(&pFs->IoctlEvent,
			                                  Executive,
			                                  KernelMode,
			                                  FALSE,
			                                  NULL);
				        }
			        else
			    	    {
			            ntStatus = pFs->IoctlStatus;
			        	}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_HIGH, ("   Waiting for request to complete\n"));

		            /* wait until complete */
		            KeWaitForSingleObject(&pFs->IoctlEvent,
		                                  Executive,
		                                  KernelMode,
		                                  FALSE,
		                                  NULL);

		            ntStatus = pFs->IoctlStatus;
					} 
				}
		    }
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate context\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
		}

    USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBIoctlRequest %08X\n", ntStatus));

    return ntStatus;
	}

/**************************************************************************
 * USBBulkCompletionRoutine
 *                                                                         
 * Completion routine for USB Bulk requests.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBBulkCompletionRoutine 
	(
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp,
    PCOMPLETION_CONTEXT Context
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	PCUSTOMPARAM pFs;
    PURB_BULK_OR_INTERRUPT_TRANSFER bulkUrb;
    KIRQL irql;


    USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBBulkCompletionRoutine\n"));

    pdoDeviceExtension = Context->DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

    /* Get a pointer to the Bulk Transfer URB in our Device Extension */
    bulkUrb = (PURB_BULK_OR_INTERRUPT_TRANSFER)Context->Urb;

    USBMASS_KdPrint(DBGLVL_HIGH, ("   Bulk Completion Status = 0x%x\n", Irp->IoStatus.Status));
    USBMASS_KdPrint(DBGLVL_HIGH, ("   Bulk Data Xfer = 0x%x\n", bulkUrb->TransferBufferLength));

    USBMASS_KdPrint(DBGLVL_HIGH, ("   Set Event\n"));

    KeSetEvent(Context->Event,
               IO_NO_INCREMENT,
               FALSE);

	pFs->BulkStatus = Irp->IoStatus.Status;
	pFs->BulkTransferLength = bulkUrb->TransferBufferLength;

    if (Context->Irp)
    	{
	    USBMASS_KdPrint(DBGLVL_HIGH, ("   Freeing Irp\n"));
    	IoFreeIrp(Context->Irp);
		Context->Irp = NULL;
		}

	if (Context->Urb)
    	{
	    USBMASS_KdPrint(DBGLVL_HIGH, ("   Freeing Urb\n"));
        ExFreePool(Context->Urb);
		Context->Urb = NULL;
	    }

    KeAcquireSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, &irql);
	    {
		pFs->BulkIrp = NULL;

	    if (!NT_SUCCESS(pFs->BulkStatus))
			{
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Bad Bulk Status = 0x%x\n", pFs->BulkStatus));
			if (pFs->IoctlIrp)
				{
			    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Cancelling Ioctl IRP\n"));
		    	IoCancelIrp(pFs->IoctlIrp);
				}
			}
	    }
    KeReleaseSpinLock(&pdoDeviceExtension->ExtensionDataSpinLock, irql);

    USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBBulkCompletionRoutine %08X\n", STATUS_MORE_PROCESSING_REQUIRED));

    return STATUS_MORE_PROCESSING_REQUIRED;
}

/**************************************************************************
 * USBBulkRequest
 *                                                                         
 * Issues a USB Bulk request.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBBulkRequest 
	(
    PDEVICE_OBJECT  DeviceObject,
    PURB            Urb,
	LONG			IrpTimeout
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	PCUSTOMPARAM pFs;
    PIRP                    irp;
    PIO_STACK_LOCATION      nextStack;
    NTSTATUS                ntStatus;
    PCOMPLETION_CONTEXT context;


    PAGED_CODE();

    USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBBulkRequest\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

    context = &pFs->bulkContext;

	if (context)
		{
	    /* Initialize the event we'll wait on */
	    KeInitializeEvent(&pFs->BulkEvent,
	                      SynchronizationEvent,
	                      FALSE);

	    /* Allocate the Irp */
	    irp = IoAllocateIrp(fdoDeviceExtension->StackDeviceObject->StackSize, FALSE);

	    if (irp == NULL)
		    {
		    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBBulkRequest %08X\n", STATUS_INSUFFICIENT_RESOURCES));
	        return STATUS_INSUFFICIENT_RESOURCES;
	    	}

	    /* Set the Irp parameters */
	    nextStack = IoGetNextIrpStackLocation(irp);
	    nextStack->MajorFunction = IRP_MJ_INTERNAL_DEVICE_CONTROL;
	    nextStack->Parameters.DeviceIoControl.IoControlCode =
	        IOCTL_INTERNAL_USB_SUBMIT_URB;
	    nextStack->Parameters.Others.Argument1 = Urb;

		context->DeviceObject = DeviceObject;
		context->Event = &pFs->BulkEvent;
		context->Urb = Urb;
		context->Irp = irp;

		pFs->BulkStatus = STATUS_SUCCESS;
		pFs->BulkIrp = irp;
		pFs->Read = Urb->UrbBulkOrInterruptTransfer.TransferFlags ? TRUE : FALSE;
		
		USBMASS_KdPrint(DBGLVL_HIGH, ("The read flag is: %d\n", pFs->Read));


	    /* Set the completion routine, which will signal the event */
	    IoSetCompletionRoutine(irp,
	                           USBBulkCompletionRoutine,
	                           context,
	                           TRUE,    // InvokeOnSuccess
	                           TRUE,    // InvokeOnError
	                           TRUE);   // InvokeOnCancel


	    /* Pass the Irp & Urb down the stack */
	    ntStatus = IoCallDriver(fdoDeviceExtension->StackDeviceObject,
	                            irp);

	    /* If the request is pending, block until it completes */
	    if (ntStatus == STATUS_PENDING)
		    {
			if (IrpTimeout)
				{
				if (IrpTimeout != IRP_INFINITE)
					{
			        LARGE_INTEGER timeout;

			        /* Specify a timeout of 5 seconds to wait for this call to complete. */
			        timeout.QuadPart = -10000 * IrpTimeout;

				    USBMASS_KdPrint(DBGLVL_HIGH, ("   Waiting for Bulk Event to complete, %dms timeout\n", IrpTimeout));
				    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);
			        ntStatus = KeWaitForSingleObject(&pFs->BulkEvent,
			                                         Executive,
			                                         KernelMode,
			                                         FALSE,
			                                         &timeout);

			        if (ntStatus == STATUS_TIMEOUT)
				        {
					    USBMASS_KdPrint(DBGLVL_HIGH, ("   Timed out... Cancelling request\n"));

			            ntStatus = STATUS_IO_TIMEOUT;

						if (irp)
							{
				            /* Cancel the Irp we just sent. */
				            IoCancelIrp(irp);
							}

					    USBMASS_KdPrint(DBGLVL_HIGH, ("   Waiting for request to complete\n"));

			            /* And wait until the cancel completes */
			            KeWaitForSingleObject(&pFs->BulkEvent,
			                                  Executive,
			                                  KernelMode,
			                                  FALSE,
			                                  NULL);
				        }
			        else
			    	    {
			            ntStatus = pFs->BulkStatus;
			        	}
					}
				else
					{
				    USBMASS_KdPrint(DBGLVL_HIGH, ("   Waiting for Bulk Event to complete, infinite\n"));
		            /* wait until complete */
		            KeWaitForSingleObject(&pFs->BulkEvent,
		                                  Executive,
		                                  KernelMode,
		                                  FALSE,
		                                  NULL);

		            ntStatus = pFs->BulkStatus;
					} 
				}
		    }
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate context\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
		}

    USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBBulkRequest %08X\n", ntStatus));

    return ntStatus;
	}

/**************************************************************************
 * USBSendSeqString
 *                                                                         
 * Issue and execute a 725 Sequencer string.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBSendSeqString
	(
    PDEVICE_OBJECT DeviceObject,
	PVOID  Buffer,
	ULONG  BufferLength,
	LONG irpTimeout
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;
	PURB urb;
    PUCHAR pch;
	USHORT Value;
	USHORT Index;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBSendSeqString\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* initialize value and index */
	Value = 0;
	Index = 0;

	/* check for input buffer length for correct size */
	if (BufferLength > 3)
		{
		/* map buffer and set value/index */
		pch = ( PUCHAR ) Buffer+2;
		Value = *pch;
		Value = Value << 8;

		pch = ( PUCHAR ) Buffer;
		Value |= *pch;


		pch = ( PUCHAR ) Buffer+1;
		Index = *pch;

		pch = ( PUCHAR ) Buffer+3;

		/* build vendor defined request */
		urb = USBBuildVendorOrClassRequest							
					(													
						URB_FUNCTION_VENDOR_DEVICE,						
						USBD_TRANSFER_DIRECTION_OUT,
						(BufferLength - 3),							
						pch,									
						0x40,
						SEQ_LOAD,								
						Value,											
						Index												
					);

		if (urb)
			{
			ntStatus = USBIoctlRequest(DeviceObject, urb, irpTimeout);
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
	    	}
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Invalid buffer length\n"));
		ntStatus = STATUS_INVALID_PARAMETER;
		}


	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBSendSeqString, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}

/**************************************************************************
 * USBSendSoftReset
 *                                                                         
 * Issue 725 Soft Reset request.
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBSendSoftReset
	(
    PDEVICE_OBJECT DeviceObject
    )
	{
	NTSTATUS ntStatus = STATUS_SUCCESS;
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PURB urb;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBSendSoftReset\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* build vendor defined request */
	urb = USBBuildVendorOrClassRequest							
				(													
				URB_FUNCTION_CLASS_OTHER,
				0,
				0,							
				NULL,									
				0x21,
				SOFT_RESET,
				0,											
				0												
				);

	if (urb)
		{
		ntStatus = USBIoctlRequest(DeviceObject, urb, IOCTL_TIMEOUT);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
    	}


	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBSendSoftReset, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}

/**************************************************************************
 * USBSendSoftReset
 *                                                                         
 * Issue a Abort Sequencer request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBAbortSeq
	(
    PDEVICE_OBJECT DeviceObject
    )
	{
	NTSTATUS ntStatus = STATUS_SUCCESS;
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PURB urb;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBAbortSeq\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* build vendor defined request */
	urb = USBBuildVendorOrClassRequest							
				(													
				URB_FUNCTION_VENDOR_DEVICE,						
				0,
				0,							
				NULL,									
				0x40,
				SEQ_ABORT,								
				0,											
				0												
				);

	if (urb)
		{
		ntStatus = USBIoctlRequest(DeviceObject, urb, IOCTL_TIMEOUT);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
    	}


	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBAbortSeq, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * USBGetSeqStatus
 *                                                                         
 * Issue a Get Sequencer Status request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBGetSeqStatus
	(
    PDEVICE_OBJECT DeviceObject,
	PSEQUENCER_STATUS pSeqStatus
    )
	{
	NTSTATUS ntStatus = STATUS_SUCCESS;
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PURB urb;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBGetSeqStatus\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* invalidate seq position */
	pSeqStatus->seqPos = 0xFF;

	/* build vendor defined request */
	urb = USBBuildVendorOrClassRequest							
				(													
				URB_FUNCTION_VENDOR_DEVICE,						
				USBD_TRANSFER_DIRECTION_IN | USBD_SHORT_TRANSFER_OK,
				sizeof(SEQUENCER_STATUS),							
				pSeqStatus,									
				0xc0,
				GET_SEQ_STATUS,								
				0,											
				0												
				);

	if (urb)
		{
		ntStatus = USBIoctlRequest(DeviceObject, urb, IOCTL_TIMEOUT);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
    	}


	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBGetSeqStatus, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * USBModifyRegister
 *                                                                         
 * Modify a specific 725 Register
 *
 * RETURNS:
 *    TRUE if successful, FALSE otherwise.
 */                                                                        
BOOLEAN
USBModifyRegister
	(
    PDEVICE_OBJECT  DeviceObject,
	UCHAR			address,
	UCHAR			value,
	UCHAR 			mask
    )
	{
    PPDO_DEVICE_EXTENSION   pdoDeviceExtension;
    NTSTATUS                ntStatus;
	BOOLEAN					success = TRUE;
	PCUSTOMPARAM 				pFs;
	UCHAR seqMod725Reg[] =
		{
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|address),value,mask, 			// set bit(s) of a specific address
		};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBModifyRegister\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	pFs = &pdoDeviceExtension->CustomParams;

    RtlCopyMemory(pFs->SeqBuffer,
                  seqMod725Reg,
                  sizeof(seqMod725Reg));

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					sizeof(seqMod725Reg),
					SEQ_TIMEOUT
					);

	if (ntStatus != STATUS_SUCCESS)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Seq Request seqMod725Reg failed (0x%x)\n", ntStatus));
		success = FALSE;
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBModifyRegister - %d\n", success));
    return success;
	}


/**************************************************************************
 * USBWriteRegister
 *                                                                         
 * Issue a Write Register request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBWriteRegister
	(
    PDEVICE_OBJECT DeviceObject,
	UCHAR Register,
	UCHAR Value
    )
	{
	NTSTATUS ntStatus = STATUS_SUCCESS;
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PURB urb;
	USHORT wValue;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBWriteRegister\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* set wValue */
	wValue = 0x0000;
	wValue = Register;
	wValue = wValue << 8;

	wValue |= Value;

	/* build vendor defined request */
	urb = USBBuildVendorOrClassRequest							
				(													
				URB_FUNCTION_VENDOR_DEVICE,						
				0,
				0,							
				NULL,									
				0x40,
				SET_1284_REGISTER,								
				wValue,											
				0												
				);

	if (urb)
		{
		ntStatus = USBIoctlRequest(DeviceObject, urb, IOCTL_TIMEOUT);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
    	}


	if (NT_SUCCESS(ntStatus))
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Write Register = 0x%x, Value = 0x%x\n", Register, Value));
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Write Register failed, Status = 0x%X\n", ntStatus));
		}


	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBWriteRegister, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}

/**************************************************************************
 * USBReadRegister
 *                                                                         
 * Issue a Read Register request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBReadRegister
	(
    PDEVICE_OBJECT DeviceObject,
	UCHAR Register,
	PUCHAR Value
    )
	{
	NTSTATUS ntStatus = STATUS_SUCCESS;
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
	PVOID TransferBuffer;
	ULONG TransferBufferLength;
	PURB urb;
    PUCHAR pch;
	USHORT wValue;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBReadRegister\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

	/* initialize register value */
	*Value = 0x00;

	/* set wValue */
	wValue = 0x0000;
	wValue = Register;
	wValue = wValue << 8;

	/* check if the output buffer length is enough */
	TransferBufferLength = NUMBER_OF_REGS_R;

	/* allocate memory to temporary transfer buffer */
	TransferBuffer = ExAllocatePoolWithTag(NonPagedPool,
		                                    TransferBufferLength,
		                                    POOL_TAG);

	/* check if we got the valid pointer */
	if (TransferBuffer)
		{
		/* build vendor defined request */
		urb = USBBuildVendorOrClassRequest							
					(													
					URB_FUNCTION_VENDOR_DEVICE,						
					USBD_TRANSFER_DIRECTION_IN | USBD_SHORT_TRANSFER_OK,
					TransferBufferLength,							
					TransferBuffer,									
					0xc0,
					GET_1284_REGISTER,								
					wValue,											
					0												
					);

		if (urb)
			{
	        ntStatus = USBMASS_SyncSendUsbRequest(pdoDeviceExtension->ParentFDO, urb);

			if (NT_SUCCESS(ntStatus))
				{
				switch(Register)
					{
					case REG_STAT:
						pch = ( PUCHAR ) TransferBuffer;
						*Value = *pch;
						break;
					case REG_CTRL:
						pch = ( PUCHAR ) TransferBuffer+1;
						*Value = *pch;
						break;
					case REG_ECRR:
						pch = ( PUCHAR ) TransferBuffer+2;
						*Value = *pch;
						break;
					case REG_CCTR:
						pch = ( PUCHAR ) TransferBuffer+3;
						*Value = *pch;
						break;
					case REG_DATA:
						pch = ( PUCHAR ) TransferBuffer+4;
						*Value = *pch;
						break;
					case REG_EPPD:
						pch = ( PUCHAR ) TransferBuffer+5;
						*Value = *pch;
						break;
					case REG_SETU:
						pch = ( PUCHAR ) TransferBuffer+6;
						*Value = *pch;
						break;
					case REG_TIMO:
						pch = ( PUCHAR ) TransferBuffer+7;
						*Value = *pch;
						break;
					case REG_ATAA:
						pch = ( PUCHAR ) TransferBuffer+8;
						*Value = *pch;
						break;
					}
				}
	        ExFreePool(urb);
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate urb\n"));
			ntStatus = STATUS_INSUFFICIENT_RESOURCES;
	    	}

	    ExFreePool(TransferBuffer);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate TransferBuffer\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
		}

	if (NT_SUCCESS(ntStatus))
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Read Register = 0x%x, Value = 0x%x\n", Register, *Value));
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Read Register failed, Status = 0x%X\n", ntStatus));
		}

	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBReadRegister, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}

/**************************************************************************
 * USBReadBulk
 *                                                                         
 * Issue a Read Bulk data request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBReadBulk
	(
    PDEVICE_OBJECT DeviceObject,
	PVOID Buffer,
	PMDL Mdl,
	ULONG BufferLength,
	LONG irpTimeout
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;
	PURB urb;
    USBD_PIPE_HANDLE pipeHandle;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBReadBulk\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pipeHandle = fdoDeviceExtension->BulkInPipe->PipeHandle;

	/* build vendor defined request */
	urb = USBBuildBulkRequest							
				(													
					BufferLength,							
					Buffer,									
					Mdl,
					pipeHandle,
					TRUE
				);

	if (urb)
		{
		ntStatus = USBBulkRequest(DeviceObject, urb, irpTimeout);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate Urb\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
		}

	if (NT_SUCCESS(ntStatus))
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Read Bulk, %d bytes\n", BufferLength));
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Read Bulk failed, Status = 0x%X\n", ntStatus));
		}

	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBReadBulk, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}

/**************************************************************************
 * USBWriteBulk
 *                                                                         
 * Issue a Write Bulk data request
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBWriteBulk
	(
    PDEVICE_OBJECT DeviceObject,
	PVOID Buffer,
	PMDL Mdl,
	ULONG BufferLength,
	LONG irpTimeout
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;
	PURB urb;
    USBD_PIPE_HANDLE pipeHandle;


	USBMASS_KdPrint(DBGLVL_HIGH, ("Entering USBWriteBulk\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pipeHandle = fdoDeviceExtension->BulkOutPipe->PipeHandle;

	/* build vendor defined request */
	urb = USBBuildBulkRequest							
				(													
					BufferLength,							
					Buffer,
					Mdl,
					pipeHandle,
					FALSE
				);

	if (urb)
		{
		ntStatus = USBBulkRequest(DeviceObject, urb, irpTimeout);
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Could not allocate Urb\n"));
		ntStatus = STATUS_INSUFFICIENT_RESOURCES;
		}

	if (NT_SUCCESS(ntStatus))
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Write Bulk, %d bytes\n", BufferLength));
		}
	else
		{
		USBMASS_KdPrint(DBGLVL_MEDIUM, ("   Write Bulk failed, Status = 0x%X\n", ntStatus));
		}

	USBMASS_KdPrint(DBGLVL_HIGH, ("Leaving USBWriteBulk, Status = 0x%X\n", ntStatus));
	return ntStatus;
	}


/**************************************************************************
 * USBMASS_IssueInternalCdb
 *                                                                         
 * Issue internal Srb
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_IssueInternalCdb (
    PDEVICE_OBJECT  DeviceObject,
    PVOID           DataBuffer,
    PULONG          DataTransferLength,
    PCDB            Cdb,
    UCHAR           CdbLength,
	BOOLEAN			Read,
    ULONG           TimeOutValue,
    ULONG           RetryValue
    )
	{
    PIRP                    irp;
    PIO_STACK_LOCATION      nextStack;
    PSCSI_REQUEST_BLOCK     srb;
    PCDB                    cdb;
    PSENSE_DATA             senseInfoBuffer;
    PMDL                    mdl;
    ULONG                   retryCount;
    KEVENT                  localevent;
    NTSTATUS                ntStatus;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_IssueInternalCdb\n"));

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Recieved TransferLength = %d\n", *DataTransferLength));

    /* Initialize these so we can bail early if an allocation fails */
    ntStatus        = STATUS_INSUFFICIENT_RESOURCES;
    irp             = NULL;
    srb             = NULL;
    senseInfoBuffer = NULL;
    mdl             = NULL;

    /* Allocate the Irp including a stack location for a completion routine */
    irp = IoAllocateIrp((CCHAR)(DeviceObject->StackSize), FALSE);

    if (irp == NULL)
	    {
        goto USBMASS_InitializeData_Exit;
    	}

    /* Allocate the Srb */
    srb = ExAllocatePoolWithTag(NonPagedPool, sizeof(SCSI_REQUEST_BLOCK),
                                POOL_TAG);

    if (srb == NULL)
	    {
        goto USBMASS_InitializeData_Exit;
    	}

    /* Allocate the sense buffer */
    senseInfoBuffer = ExAllocatePoolWithTag(NonPagedPool, SENSE_BUFFER_SIZE,
                                            POOL_TAG);

    if (senseInfoBuffer == NULL)
	    {
        goto USBMASS_InitializeData_Exit;
    	}


    /* Try the request up to RetryValue times */
    for (retryCount = 0; retryCount < RetryValue; retryCount++)
	    {
        nextStack = IoGetNextIrpStackLocation(irp);
        nextStack->MajorFunction = IRP_MJ_SCSI;
        nextStack->Parameters.Scsi.Srb = srb;

        /* (Re)Initialize the Srb */
        RtlZeroMemory(srb, sizeof(SCSI_REQUEST_BLOCK));

        srb->Length     = sizeof(SCSI_REQUEST_BLOCK);
        srb->Function   = SRB_FUNCTION_EXECUTE_SCSI;
        srb->CdbLength  = CdbLength;
        srb->SrbFlags   = (Read) ? SRB_FLAGS_DATA_IN : SRB_FLAGS_DATA_OUT;

        srb->SenseInfoBufferLength  = SENSE_BUFFER_SIZE;
        srb->SenseInfoBuffer        = senseInfoBuffer;

        srb->DataTransferLength     = *DataTransferLength;
        srb->DataBuffer             = DataBuffer;

        srb->TimeOutValue = TimeOutValue;

	    USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Setting TransferLength = %d\n", srb->DataTransferLength));

        /* (Re)Initialize the Cdb */
        RtlCopyMemory(srb->Cdb, Cdb, sizeof(CDB));

        /* Initialize the MDL (first time only) */
        if (retryCount == 0)
	        {
            mdl = IoAllocateMdl(DataBuffer,
                                *DataTransferLength,
                                FALSE,
                                FALSE,
                                NULL);

            if (!mdl)
    	        {
                ntStatus = STATUS_INSUFFICIENT_RESOURCES;
                goto USBMASS_InitializeData_Exit;
        	    }

            MmBuildMdlForNonPagedPool(mdl);
	        }

        irp->MdlAddress = mdl;


        /* Initialize the event we'll wait on */
        KeInitializeEvent(&localevent,
                          SynchronizationEvent,
                          FALSE);

        /* Set the completion routine, which will signal the event */
        IoSetCompletionRoutine(irp,
                               USBMASS_SyncCompletionRoutine,
                               &localevent,
                               TRUE,    // InvokeOnSuccess
                               TRUE,    // InvokeOnError
                               TRUE);   // InvokeOnCancel

        /* Pass the Irp & Srb down the stack */
        ntStatus = IoCallDriver(DeviceObject, irp);

        /* If the request is pending, block until it completes */
        if (ntStatus == STATUS_PENDING)
	        {
            KeWaitForSingleObject(&localevent,
                                  Executive,
                                  KernelMode,
                                  FALSE,
                                  NULL);

            /* Get final completion status */
            ntStatus = irp->IoStatus.Status;
	        }

        USBMASS_KdPrint(DBGLVL_MINIMUM, ("   USBIssueInternalCdb %d %08X %08X\n",
                     retryCount, ntStatus, srb->SrbStatus));

        if (NT_SUCCESS(ntStatus) &&
            SRB_STATUS(srb->SrbStatus) == SRB_STATUS_SUCCESS)
	        {
            *DataTransferLength = srb->DataTransferLength;
            break;
    	    }
        else
        	{
            ntStatus = STATUS_UNSUCCESSFUL;
	        }
    	}

USBMASS_InitializeData_Exit:

    if (mdl != NULL)
	    {
        IoFreeMdl(mdl);
    	}

    if (senseInfoBuffer != NULL)
	    {
        ExFreePool(senseInfoBuffer);
    	}

    if (srb != NULL)
	    {
        ExFreePool(srb);
    	}

    if (irp != NULL)
	    {
        IoFreeIrp(irp);
    	}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_IssueInternalCdb %08X\n", ntStatus));

    return ntStatus;
	}



/**************************************************************************
 * USBMASS_CDBTimeoutValue
 *                                                                         
 * Used to determine the timeout required based on Device Type and SCSIOP code
 *
 * RETURNS:
 *    ULONG CDB timeout value in seconds
 */                                                                        
ULONG USBMASS_CDBTimeoutValue
	(
	PCDB cdb, 
	ULONG DeviceType
	) 
	{
	ULONG timeout = STD_CMD_TIMEOUT_MS;


	if (DeviceType == SEQUENTIAL_ACCESS_DEVICE)
		{
		/* change default to tape device default */
		timeout = TAPE_CMD_TIMEOUT_MS;

		switch (cdb->CDB6GENERIC.OperationCode)
			{
			case SCSIOP_LOAD_UNLOAD:
				timeout = 390;
				if (cdb->CDB6GENERIC.CommandUniqueBytes[2] == 0x01)
					{
					timeout = 900;
					}
				break;

			case SCSIOP_MEDIUM_REMOVAL:
				timeout = 390;
				break;

			case SCSIOP_LOCATE:
			case SCSIOP_SPACE:
				timeout = 480;
				break;

			case SCSIOP_WRITE_FILEMARKS:
				timeout = 900;
				break;

			case SCSIOP_MODE_SELECT:
				timeout = 16500;
				break;

			case SCSIOP_ERASE:
				if (cdb->ERASE.Long == SETBITON)	
					{
					timeout = 16500;
					}
				break;
			}
		}
	else if (DeviceType == READ_ONLY_DIRECT_ACCESS_DEVICE)
		{
		/* change default to cd-rom device default */
		timeout = CDROM_CMD_TIMEOUT_MS;
		}
	else if (DeviceType == OPTICAL_DEVICE)
		{
		/* change default to optical (MO) device default */
		timeout = OPTICAL_CMD_TIMEOUT_MS;

		switch (cdb->CDB6GENERIC.OperationCode)
			{
			case SCSIOP_FORMAT_UNIT:
				timeout = 3600;
				break;
			}
		}


	USBMASS_KdPrint(DBGLVL_MINIMUM, ("USBMASS_CDBTimeoutValue:  returning a timeout of %d seconds\n", timeout));
	return timeout;
	}


/**************************************************************************
 * USBMASS_Resume
 *                                                                         
 * Called after the device comes out of suspend
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_Resume 
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Resume\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	ntStatus = USBMASS_PerformReset(DeviceObject);	

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Resume %08X\n", ntStatus));

    return ntStatus;
	}


/**************************************************************************
 * USBMASS_Suspend
 *                                                                         
 * Called before the device goes into suspend
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_Suspend 
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;
	PCUSTOMPARAM pFs;
	UCHAR	seqLength;
	static  UCHAR seqSuspendFalstaffLite[]={
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_CTRL),0x00,0x80, 			// Set RESET- bit high
		(SEQ_RRMW|REG_ECRR),0x00,0xE0, 			// Take it out of ATA Mode
        (SEQ_RRMW|REG_CTRL),0x0B,0x0F,          // Put falstaff lite into Suspend and nAutoFd & nStrobe low
        (SEQ_RRMW|REG_DATA),0x00,0xFF,          // Set data registers low
        (SEQ_RRMW|REG_ATAA),0x00,0x1F,          // Set ata CS0-1, DA0-2 registers low
	};
	static  UCHAR seqSuspenduDrive[]={
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_CTRL),0x00,0x80, 			// Set RESET- bit high
		(SEQ_RRMW|REG_ECRR),0x00,0xE0, 			// Take it out of ATA Mode								
		(SEQ_RRMW|REG_CTRL),0x07,0x07, 			// Take uDrive out of power on mode and set nAutoFd & nStrobe low
        (SEQ_RRMW|REG_DATA),0x00,0xFF,          // Set data registers low
        (SEQ_RRMW|REG_ATAA),0x00,0x1F,          // Set ata CS0-1, DA0-2 registers low
	};
	static  UCHAR seqSuspendFalstaff[]={
		0x00,0x00,SEQ_BLOCKING_BIT, 			// Loop Count (WORD) & Block bit (bit 8 of BYTE)
		(SEQ_RRMW|REG_CTRL),0x00,0x04, 			// If using nInit as reset, set RESET- bit high
		(SEQ_RRMW|REG_CTRL),0x00,0x80, 			// If using HLH as reset, set RESET- bit high
		(SEQ_RRMW|REG_ECRR),0x00,0xE0, 			// Take it out of ATA Mode								
        (SEQ_RRMW|REG_CTRL),0x03,0x03,          // Set nAutoFd & nStrobe low
        (SEQ_RRMW|REG_DATA),0x00,0xFF,          // Set data registers low
        (SEQ_RRMW|REG_ATAA),0x00,0x1F,          // Set ata CS0-1, DA0-2 registers low
	};


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_Suspend\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	pFs = &pdoDeviceExtension->CustomParams;

	if (fdoDeviceExtension->DriverFlags & DF_CUST_FALSTAFFLITE)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected Falstaff-Lite device\n"));
		seqLength = sizeof(seqSuspendFalstaffLite);
	    RtlCopyMemory(pFs->SeqBuffer,
	                  seqSuspendFalstaffLite,
	                  seqLength);
		}
	else
		{
		if (fdoDeviceExtension->DriverFlags & DF_CUST_UDRIVE)
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected uDrive device\n"));
			seqLength = sizeof(seqSuspenduDrive);
		    RtlCopyMemory(pFs->SeqBuffer,
		                  seqSuspenduDrive,
		                  seqLength);
			}
		else
			{
			USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Detected Standard Falstaff device\n"));
			seqLength = sizeof(seqSuspendFalstaff);
		    RtlCopyMemory(pFs->SeqBuffer,
		                  seqSuspendFalstaff,
		                  seqLength);
			}
		}

	ntStatus = USBSendSeqString
					(
					DeviceObject,
					pFs->SeqBuffer,
					seqLength,
					SEQ_TIMEOUT
					);

	if (ntStatus != STATUS_SUCCESS)
		{
		USBMASS_KdPrint(DBGLVL_MINIMUM, ("   Suspend Sequence failed\n"));
		}

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_Suspend %08X\n", ntStatus));

    return ntStatus;
	}


/**************************************************************************
 * USBMASS_DeviceWasRemoved
 *                                                                         
 * Called if the device was disabled/stopped and is about to be started again
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_DeviceWasRemoved 
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_DeviceWasRemoved\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	ntStatus = USBMASS_PerformReset(DeviceObject);	

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_DeviceWasRemoved %08X\n", ntStatus));

    return ntStatus;
	}


/**************************************************************************
 * USBMASS_StopDevice
 *                                                                         
 * Called if the device is going to be disabled/stopped
 *
 * RETURNS:
 *    NT status code
 */                                                                        
NTSTATUS
USBMASS_StopDevice 
	(
    PDEVICE_OBJECT  DeviceObject
    )
	{
    PPDO_DEVICE_EXTENSION pdoDeviceExtension;
    PFDO_DEVICE_EXTENSION fdoDeviceExtension;
	NTSTATUS ntStatus = STATUS_SUCCESS;


    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Entering USBMASS_StopDevice\n"));

    pdoDeviceExtension = DeviceObject->DeviceExtension;
    ASSERT(pdoDeviceExtension->Type == USBMASS_DO_TYPE_PDO);

    fdoDeviceExtension = pdoDeviceExtension->ParentFDOExtension;
    ASSERT(fdoDeviceExtension->Type == USBMASS_DO_TYPE_FDO);

	ntStatus = USBMASS_Suspend(DeviceObject);

    USBMASS_KdPrint(DBGLVL_MINIMUM, ("Leaving USBMASS_StopDevice %08X\n", ntStatus));

    return ntStatus;
	}
